@@ -1,5 +1,6 @@
 /*
- * Copyright (C) Internet Systems Consortium, Inc. ("ISC")
+ * Copyright (C) 2004-2014  Internet Systems Consortium, Inc. ("ISC")
+ * Copyright (C) 1999-2003  Internet Software Consortium.
  *
  * Permission to use, copy, modify, and/or distribute this software for any
  * purpose with or without fee is hereby granted, provided that the above
@@ -14,6 +15,8 @@
  * PERFORMANCE OF THIS SOFTWARE.
  */
 
+/* $Id: update.c,v 1.199 2011/12/22 07:32:40 each Exp $ */
+
 #include <config.h>
 
 #include <isc/netaddr.h>
@@ -810,11 +813,8 @@ typedef struct {
 	/* The signature's name if the request was signed. */
 	dns_name_t *signer;
 
-	/* The address of the client. */
-	isc_netaddr_t *addr;
-
-	/* Whether the request was sent via TCP. */
-	isc_boolean_t tcp;
+	/* The address of the client if the request was received via TCP. */
+	isc_netaddr_t *tcpaddr;
 
 	/* The ssu table to check against. */
 	dns_ssutable_t *table;
@@ -835,17 +835,16 @@ ssu_checkrule(void *data, dns_rdataset_t *rrset) {
 	if (rrset->type == dns_rdatatype_rrsig ||
 	    rrset->type == dns_rdatatype_nsec)
 		return (ISC_R_SUCCESS);
-	result = dns_ssutable_checkrules2(ssuinfo->table, ssuinfo->signer,
-					  ssuinfo->name, ssuinfo->addr,
-					  ssuinfo->tcp, &ns_g_server->aclenv,
-					  rrset->type, ssuinfo->key);
+	result = dns_ssutable_checkrules(ssuinfo->table, ssuinfo->signer,
+					 ssuinfo->name, ssuinfo->tcpaddr,
+					 rrset->type, ssuinfo->key);
 	return (result == ISC_TRUE ? ISC_R_SUCCESS : ISC_R_FAILURE);
 }
 
 static isc_boolean_t
 ssu_checkall(dns_db_t *db, dns_dbversion_t *ver, dns_name_t *name,
 	     dns_ssutable_t *ssutable, dns_name_t *signer,
-	     isc_netaddr_t *addr, isc_boolean_t tcp, dst_key_t *key)
+	     isc_netaddr_t *tcpaddr, dst_key_t *key)
 {
 	isc_result_t result;
 	ssu_check_t ssuinfo;
@@ -853,8 +852,7 @@ ssu_checkall(dns_db_t *db, dns_dbversion_t *ver, dns_name_t *name,
 	ssuinfo.name = name;
 	ssuinfo.table = ssutable;
 	ssuinfo.signer = signer;
-	ssuinfo.addr = addr;
-	ssuinfo.tcp = tcp;
+	ssuinfo.tcpaddr = tcpaddr;
 	ssuinfo.key = key;
 	result = foreach_rrset(db, ver, name, ssu_checkrule, &ssuinfo);
 	return (ISC_TF(result == ISC_R_SUCCESS));
@@ -1685,7 +1683,7 @@ check_mx(ns_client_t *client, dns_zone_t *zone,
 		dns_name_format(&mx.mx, namebuf, sizeof(namebuf));
 		dns_name_format(&t->name, ownerbuf, sizeof(ownerbuf));
 		isaddress = ISC_FALSE;
-		if ((options & DNS_ZONEOPT_CHECKMX) != 0 &&
+		if ((options & DNS_RDATA_CHECKMX) != 0 &&
 		    strlcpy(tmp, namebuf, sizeof(tmp)) < sizeof(tmp)) {
 			if (tmp[strlen(tmp) - 1] == '.')
 				tmp[strlen(tmp) - 1] = '\0';
@@ -1694,7 +1692,7 @@ check_mx(ns_client_t *client, dns_zone_t *zone,
 				isaddress = ISC_TRUE;
 		}
 
-		if (isaddress && (options & DNS_ZONEOPT_CHECKMXFAIL) != 0) {
+		if (isaddress && (options & DNS_RDATA_CHECKMXFAIL) != 0) {
 			update_log(client, zone, ISC_LOG_ERROR,
 				   "%s/MX: '%s': %s",
 				   ownerbuf, namebuf,
@@ -2436,7 +2434,6 @@ update_action(isc_task_t *task, isc_event_t *event) {
 	update_event_t *uev = (update_event_t *) event;
 	dns_zone_t *zone = uev->zone;
 	ns_client_t *client = (ns_client_t *)event->ev_arg;
-
 	isc_result_t result;
 	dns_db_t *db = NULL;
 	dns_dbversion_t *oldver = NULL;
@@ -2452,13 +2449,12 @@ update_action(isc_task_t *task, isc_event_t *event) {
 	dns_ssutable_t *ssutable = NULL;
 	dns_fixedname_t tmpnamefixed;
 	dns_name_t *tmpname = NULL;
-	unsigned int options;
+	unsigned int options, options2;
 	dns_difftuple_t *tuple;
 	dns_rdata_dnskey_t dnskey;
 	isc_boolean_t had_dnskey;
 	dns_rdatatype_t privatetype = dns_zone_getprivatetype(zone);
-	isc_uint32_t maxrecords;
-	isc_uint64_t records;
+	dns_ttl_t maxttl = 0;
 
 	INSIST(event->ev_type == DNS_EVENT_UPDATE);
 
@@ -2688,33 +2684,38 @@ update_action(isc_task_t *task, isc_event_t *event) {
 		}
 
 		if (ssutable != NULL) {
-			isc_netaddr_t netaddr;
+			isc_netaddr_t *tcpaddr, netaddr;
 			dst_key_t *tsigkey = NULL;
-			isc_netaddr_fromsockaddr(&netaddr, &client->peeraddr);
+			/*
+			 * If this is a TCP connection then pass the
+			 * address of the client through for tcp-self
+			 * and 6to4-self otherwise pass NULL.  This
+			 * provides weak address based authentication.
+			 */
+			if (TCPCLIENT(client)) {
+				isc_netaddr_fromsockaddr(&netaddr,
+							 &client->peeraddr);
+				tcpaddr = &netaddr;
+			} else
+				tcpaddr = NULL;
 
 			if (client->message->tsigkey != NULL)
 				tsigkey = client->message->tsigkey->key;
 
 			if (rdata.type != dns_rdatatype_any) {
-				if (!dns_ssutable_checkrules2
-				    (ssutable, client->signer, name, &netaddr,
-				     ISC_TF(TCPCLIENT(client)),
-				     &ns_g_server->aclenv,
-				     rdata.type, tsigkey))
-				{
+				if (!dns_ssutable_checkrules(ssutable,
+							     client->signer,
+							     name, tcpaddr,
+							     rdata.type,
+							     tsigkey))
 					FAILC(DNS_R_REFUSED,
 					      "rejected by secure update");
-				}
 			} else {
 				if (!ssu_checkall(db, ver, name, ssutable,
-						  client->signer,
-						  &netaddr,
-						  ISC_TF(TCPCLIENT(client)),
+						  client->signer, tcpaddr,
 						  tsigkey))
-				{
 					FAILC(DNS_R_REFUSED,
 					      "rejected by secure update");
-				}
 			}
 		}
 	}
@@ -2729,6 +2730,7 @@ update_action(isc_task_t *task, isc_event_t *event) {
 	 */
 
 	options = dns_zone_getoptions(zone);
+	options2 = dns_zone_getoptions2(zone);
 	for (result = dns_message_firstname(request, DNS_SECTION_UPDATE);
 	     result == ISC_R_SUCCESS;
 	     result = dns_message_nextname(request, DNS_SECTION_UPDATE))
@@ -2745,8 +2747,7 @@ update_action(isc_task_t *task, isc_event_t *event) {
 		if (update_class == zoneclass) {
 
 			/*
-			 * RFC1123 doesn't allow MF and MD in master zones.
-			 */
+			 * RFC1123 doesn't allow MF and MD in master zones.				 */
 			if (rdata.type == dns_rdatatype_md ||
 			    rdata.type == dns_rdatatype_mf) {
 				char typebuf[DNS_RDATATYPE_FORMATSIZE];
@@ -2835,9 +2836,7 @@ update_action(isc_task_t *task, isc_event_t *event) {
 				 * Ignore attempts to add NSEC3PARAM records
 				 * with any flags other than OPTOUT.
 				 */
-				if ((rdata.data[1] &
-				     ~DNS_NSEC3FLAG_OPTOUT) != 0)
-				{
+				if ((rdata.data[1] & ~DNS_NSEC3FLAG_OPTOUT) != 0) {
 					update_log(client, zone,
 						   LOGLEVEL_PROTOCOL,
 						   "attempt to add NSEC3PARAM "
@@ -2857,16 +2856,45 @@ update_action(isc_task_t *task, isc_event_t *event) {
 					   "a non-terminal wildcard", namestr);
 			}
 
+			if ((options2 & DNS_ZONEOPT2_CHECKTTL) != 0) {
+				maxttl = dns_zone_getmaxttl(zone);
+				if (ttl > maxttl) {
+					ttl = maxttl;
+					update_log(client, zone,
+						   LOGLEVEL_PROTOCOL,
+						   "reducing TTL to the "
+						   "configured max-zone-ttl %d",
+						   maxttl);
+				}
+			}
+
 			if (isc_log_wouldlog(ns_g_lctx, LOGLEVEL_PROTOCOL)) {
 				char namestr[DNS_NAME_FORMATSIZE];
 				char typestr[DNS_RDATATYPE_FORMATSIZE];
-				dns_name_format(name, namestr,
-						sizeof(namestr));
+				char rdstr[2048];
+				isc_buffer_t buf;
+				int len = 0;
+				const char *truncated = "";
+
+				dns_name_format(name, namestr, sizeof(namestr));
 				dns_rdatatype_format(rdata.type, typestr,
 						     sizeof(typestr));
+				isc_buffer_init(&buf, rdstr, sizeof(rdstr));
+				result = dns_rdata_totext(&rdata, NULL, &buf);
+				if (result == ISC_R_NOSPACE) {
+					len = (int)isc_buffer_usedlength(&buf);
+					truncated = " [TRUNCATED]";
+				} else if (result != ISC_R_SUCCESS) {
+					snprintf(rdstr, sizeof(rdstr), "[dns_"
+						 "rdata_totext failed: %s]",
+						 dns_result_totext(result));
+					len = strlen(rdstr);
+				} else
+					len = (int)isc_buffer_usedlength(&buf);
 				update_log(client, zone, LOGLEVEL_PROTOCOL,
-					   "adding an RR at '%s' %s",
-					   namestr, typestr);
+					   "adding an RR at '%s' %s %.*s%s",
+					   namestr, typestr, len, rdstr,
+					   truncated);
 			}
 
 			/* Prepare the affected RRset for the addition. */
@@ -2889,18 +2917,10 @@ update_action(isc_task_t *task, isc_event_t *event) {
 					dns_diff_clear(&ctx.del_diff);
 					dns_diff_clear(&ctx.add_diff);
 				} else {
-					result = do_diff(&ctx.del_diff, db, ver,
-							 &diff);
-					if (result == ISC_R_SUCCESS) {
-						result = do_diff(&ctx.add_diff,
-								 db, ver,
-								 &diff);
-					}
-					if (result != ISC_R_SUCCESS) {
-						dns_diff_clear(&ctx.del_diff);
-						dns_diff_clear(&ctx.add_diff);
-						goto failure;
-					}
+					CHECK(do_diff(&ctx.del_diff, db, ver,
+						      &diff));
+					CHECK(do_diff(&ctx.add_diff, db, ver,
+						      &diff));
 					CHECK(update_one_rr(db, ver, &diff,
 							    DNS_DIFFOP_ADD,
 							    name, ttl, &rdata));
@@ -3019,19 +3039,6 @@ update_action(isc_task_t *task, isc_event_t *event) {
 			goto failure;
 		}
 	}
-	if (! ISC_LIST_EMPTY(diff.tuples)) {
-		result = dns_zone_cdscheck(zone, db, ver);
-		if (result == DNS_R_BADCDS || result == DNS_R_BADCDNSKEY) {
-			update_log(client, zone, LOGLEVEL_PROTOCOL,
-				   "update rejected: bad %s RRset",
-				   result == DNS_R_BADCDS ? "CDS" : "CDNSKEY");
-			result = DNS_R_REFUSED;
-			goto failure;
-		}
-		if (result != ISC_R_SUCCESS)
-			goto failure;
-
-	}
 
 	/*
 	 * If any changes were made, increment the SOA serial number,
@@ -3062,9 +3069,10 @@ update_action(isc_task_t *task, isc_event_t *event) {
 #define ALLOW_SECURE_TO_INSECURE(zone) \
 	((dns_zone_getoptions(zone) & DNS_ZONEOPT_SECURETOINSECURE) != 0)
 
-		CHECK(rrset_exists(db, oldver, zonename, dns_rdatatype_dnskey,
-				   0, &had_dnskey));
 		if (!ALLOW_SECURE_TO_INSECURE(zone)) {
+			CHECK(rrset_exists(db, oldver, zonename,
+					   dns_rdatatype_dnskey, 0,
+					   &had_dnskey));
 			if (had_dnskey && !has_dnskey) {
 				update_log(client, zone, LOGLEVEL_PROTOCOL,
 					   "update rejected: all DNSKEY "
@@ -3110,20 +3118,6 @@ update_action(isc_task_t *task, isc_event_t *event) {
 			}
 		}
 
-		maxrecords = dns_zone_getmaxrecords(zone);
-		if (maxrecords != 0U) {
-			result = dns_db_getsize(db, ver, &records, NULL);
-			if (result == ISC_R_SUCCESS && records > maxrecords) {
-				update_log(client, zone, ISC_LOG_ERROR,
-					   "records in zone (%"
-					   ISC_PRINT_QUADFORMAT
-					   "u) exceeds max-records (%u)",
-					   records, maxrecords);
-				result = DNS_R_TOOMANYRECORDS;
-				goto failure;
-			}
-		}
-
 		journalfile = dns_zone_getjournal(zone);
 		if (journalfile != NULL) {
 			update_log(client, zone, LOGLEVEL_DEBUG,
@@ -3268,8 +3262,6 @@ update_action(isc_task_t *task, isc_event_t *event) {
 	uev->ev_type = DNS_EVENT_UPDATEDONE;
 	uev->ev_action = updatedone_action;
 	isc_task_send(client->task, &event);
-
-	INSIST(ver == NULL);
 	INSIST(event == NULL);
 }
 
