@@ -1,9 +1,18 @@
 /*
- * Copyright (C) 1999-2001, 2004, 2005, 2007-2009, 2011-2014, 2016, 2017  Internet Systems Consortium, Inc. ("ISC")
+ * Copyright (C) 2004, 2005, 2007-2009, 2011-2014  Internet Systems Consortium, Inc. ("ISC")
+ * Copyright (C) 1999-2001  Internet Software Consortium.
  *
- * This Source Code Form is subject to the terms of the Mozilla Public
- * License, v. 2.0. If a copy of the MPL was not distributed with this
- * file, You can obtain one at http://mozilla.org/MPL/2.0/.
+ * Permission to use, copy, modify, and/or distribute this software for any
+ * purpose with or without fee is hereby granted, provided that the above
+ * copyright notice and this permission notice appear in all copies.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS" AND ISC DISCLAIMS ALL WARRANTIES WITH
+ * REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
+ * AND FITNESS.  IN NO EVENT SHALL ISC BE LIABLE FOR ANY SPECIAL, DIRECT,
+ * INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
+ * LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE
+ * OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
+ * PERFORMANCE OF THIS SOFTWARE.
  */
 
 /* $Id: t_dst.c,v 1.60 2011/03/17 23:47:29 tbox Exp $ */
@@ -26,7 +35,6 @@
 #include <isc/entropy.h>
 #include <isc/file.h>
 #include <isc/mem.h>
-#include <isc/print.h>
 #include <isc/region.h>
 #include <isc/stdio.h>
 #include <isc/string.h>
@@ -381,8 +389,7 @@ io(dns_name_t *name, isc_uint16_t id, isc_uint16_t alg, int type,
 	cleandir(tmp);
 
  failure:
-	if (key != NULL)
-		dst_key_free(&key);
+	dst_key_free(&key);
 }
 
 static void
@@ -648,12 +655,13 @@ sig_fromfile(char *path, isc_buffer_t *iscbuf) {
 
 	len = (size_t)size;
 	p = buf;
-	while (len != 0U) {
+	while (len) {
 		isc_result = isc_stdio_read(p, 1, len, fp, &rval);
 		if (isc_result == ISC_R_SUCCESS) {
 			len -= rval;
 			p += rval;
-		} else {
+		}
+		else {
 			t_info("read failed %d, result: %s\n",
 			       (int)rval, isc_result_totext(isc_result));
 			(void) free(buf);
@@ -665,38 +673,29 @@ sig_fromfile(char *path, isc_buffer_t *iscbuf) {
 
 	p = buf;
 	len = size;
-	while (len > 0U) {
+	while(len) {
 		if ((*p == '\r') || (*p == '\n')) {
 			++p;
 			--len;
 			continue;
-		} else if (len < 2U)
-		       goto err;
+		}
 		if (('0' <= *p) && (*p <= '9'))
 			val = *p - '0';
-		else if (('A' <= *p) && (*p <= 'F'))
-			val = *p - 'A' + 10;
 		else
-			goto err;
+			val = *p - 'A' + 10;
 		++p;
 		val <<= 4;
 		--len;
 		if (('0' <= *p) && (*p <= '9'))
 			val |= (*p - '0');
-		else if (('A' <= *p) && (*p <= 'F'))
-			val |= (*p - 'A' + 10);
 		else
-			goto err;
+			val |= (*p - 'A' + 10);
 		++p;
 		--len;
 		isc_buffer_putuint8(iscbuf, val);
 	}
 	(void) free(buf);
 	return(0);
-
- err:
-	(void) free(buf);
-	return (1);
 }
 
 static void
@@ -912,42 +911,9 @@ t2_sigchk(char *datapath, char *sigpath, char *keyname,
  * signed at some earlier time, possibly with an entire different
  * version or implementation of the DSA and RSA algorithms
  */
-
-isc_mem_t *t2_mctx = NULL;
-isc_entropy_t *t2_ectx = NULL;
-
-static int
-t2_vfy_init(void) {
-	isc_result_t	isc_result;
-
-	t2_mctx = NULL;
-	isc_result = isc_mem_create(0, 0, &t2_mctx);
-	if (isc_result != ISC_R_SUCCESS) {
-		t_info("isc_mem_create failed %s\n",
-		       isc_result_totext(isc_result));
-		return(0);
-	}
-	t2_ectx = NULL;
-	isc_result = isc_entropy_create(t2_mctx, &t2_ectx);
-	if (isc_result != ISC_R_SUCCESS) {
-		t_info("isc_entropy_create failed %s\n",
-		       isc_result_totext(isc_result));
-		return(0);
-	}
-	isc_result = isc_entropy_createfilesource(t2_ectx, "randomfile");
-	if (isc_result != ISC_R_SUCCESS) {
-		t_info("isc_entropy_create failed %s\n",
-		       isc_result_totext(isc_result));
-		return(0);
-	}
-	isc_result = dst_lib_init(t2_mctx, t2_ectx, ISC_ENTROPY_BLOCKING);
-	if (isc_result != ISC_R_SUCCESS) {
-		t_info("dst_lib_init failed %s\n",
-		       isc_result_totext(isc_result));
-		return(0);
-	}
-	return(1);
-}
+static const char *a2 =
+		"the dst module provides the capability to "
+		"verify data signed with the RSA and DSA algorithms";
 
 /*
  * av ==  datafile, sigpath, keyname, keyid, alg, exp_result.
@@ -964,6 +930,9 @@ t2_vfy(char **av) {
 	char		*exp_result;
 	int		nfails;
 	int		nprobs;
+	isc_mem_t	*mctx;
+	isc_entropy_t	*ectx;
+	isc_result_t	isc_result;
 	int		result;
 
 	datapath	= *av++;
@@ -985,7 +954,35 @@ t2_vfy(char **av) {
 		return(T_UNRESOLVED);
 	}
 
+	mctx = NULL;
+	isc_result = isc_mem_create(0, 0, &mctx);
+	if (isc_result != ISC_R_SUCCESS) {
+		t_info("isc_mem_create failed %s\n",
+		       isc_result_totext(isc_result));
+		return(T_UNRESOLVED);
+	}
+	ectx = NULL;
+	isc_result = isc_entropy_create(mctx, &ectx);
+	if (isc_result != ISC_R_SUCCESS) {
+		t_info("isc_entropy_create failed %s\n",
+		       isc_result_totext(isc_result));
+		return(T_UNRESOLVED);
+	}
+	isc_result = isc_entropy_createfilesource(ectx, "randomfile");
+	if (isc_result != ISC_R_SUCCESS) {
+		t_info("isc_entropy_create failed %s\n",
+		       isc_result_totext(isc_result));
+		return(T_UNRESOLVED);
+	}
+	isc_result = dst_lib_init(mctx, ectx, ISC_ENTROPY_BLOCKING);
+	if (isc_result != ISC_R_SUCCESS) {
+		t_info("dst_lib_init failed %s\n",
+		       isc_result_totext(isc_result));
+		return(T_UNRESOLVED);
+	}
+
 	if (!dst_algorithm_supported(DST_ALG_RSAMD5)) {
+		dst_lib_destroy();
 		t_info("library built without crypto support\n");
 		return (T_SKIPPED);
 	}
@@ -994,9 +991,15 @@ t2_vfy(char **av) {
 			datapath, sigpath, keyname, key, alg, exp_result);
 	t2_sigchk(datapath, sigpath, keyname, keyid,
 			algid, DST_TYPE_PRIVATE|DST_TYPE_PUBLIC,
-			t2_mctx, exp_result,
+			mctx, exp_result,
 			&nfails, &nprobs);
 
+	dst_lib_destroy();
+
+	isc_entropy_detach(&ectx);
+
+	isc_mem_destroy(&mctx);
+
 	result = T_UNRESOLVED;
 	if (nfails)
 		result = T_FAIL;
@@ -1006,24 +1009,11 @@ t2_vfy(char **av) {
 	return(result);
 }
 
-static const char *a2 =
-		"the dst module provides the capability to "
-		"verify data signed with the RSA and DSA algorithms";
-
 static void
 t2(void) {
 	int	result;
 	t_assert("dst", 2, T_REQUIRED, "%s", a2);
-	if (!t2_vfy_init()) {
-		result = T_UNRESOLVED;
-	} else {
-		result = t_eval("dst_2_data", t2_vfy, 6);
-		dst_lib_destroy();
-	}
-	if (t2_ectx)
-		isc_entropy_detach(&t2_ectx);
-	if (t2_mctx)
-		isc_mem_destroy(&t2_mctx);
+	result = t_eval("dst_2_data", t2_vfy, 6);
 	t_result(result);
 }
 
