@@ -1,6 +1,7 @@
 #!/bin/sh
 #
-# Copyright (C) Internet Systems Consortium, Inc. ("ISC")
+# Copyright (C) 2004, 2007, 2011-2014  Internet Systems Consortium, Inc. ("ISC")
+# Copyright (C) 2001  Internet Software Consortium.
 #
 # Permission to use, copy, modify, and/or distribute this software for any
 # purpose with or without fee is hereby granted, provided that the above
@@ -14,15 +15,16 @@
 # OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 # PERFORMANCE OF THIS SOFTWARE.
 
+# $Id: tests.sh,v 1.10 2011/09/01 05:28:14 marka Exp $
+
 SYSTEMTESTTOP=..
 . $SYSTEMTESTTOP/conf.sh
 
 status=0
-n=0
 
-RNDCOPTS="-c ../common/rndc.conf -s 10.53.0.2 -p ${CONTROLPORT}"
+RNDCOPTS="-c ../common/rndc.conf -s 10.53.0.2 -p 9953"
 DIGOPTS="+nosea +nocomm +nocmd +noquest +noadd +noauth +nocomm \
-         +nostat @10.53.0.2 -p ${PORT}"
+         +nostat @10.53.0.2 -p 5300"
 
 # fill the cache with nodes from flushtest.example zone
 load_cache () {
@@ -62,13 +64,9 @@ EOF
 }
 
 dump_cache () {
-        $RNDC $RNDCOPTS dumpdb -cache
-        for i in 0 1 2 3 4 5 6 7 8 9
-        do
-                grep '^; Dump complete$' ns2/named_dump.db > /dev/null && break
-                sleep 1
-        done
-        mv ns2/named_dump.db ns2/named_dump.db.$n
+        rm -f ns2/named_dump.db
+        $RNDC $RNDCOPTS dumpdb -cache _default
+        sleep 1
 }
 
 clear_cache () {
@@ -86,40 +84,36 @@ in_cache () {
         return 0
 }
 
-n=`expr $n + 1`
-echo_i "check correctness of routine cache cleaning ($n)"
+echo "I:check correctness of routine cache cleaning"
 $DIG $DIGOPTS +tcp +keepopen -b 10.53.0.7 -f dig.batch > dig.out.ns2 || status=1
+grep ";" dig.out.ns2
 
-digcomp --lc dig.out.ns2 knowngood.dig.out || status=1
+$PERL ../digcomp.pl --lc dig.out.ns2 knowngood.dig.out || status=1
 
-n=`expr $n + 1`
-echo_i "only one tcp socket was used ($n)"
+echo "I:only one tcp socket was used"
 tcpclients=`grep "client 10.53.0.7#[0-9]*:" ns2/named.run | awk '{print $4}' | sort | uniq -c | wc -l`
 
-test $tcpclients -eq 1 || { status=1; echo_i "failed"; }
+test $tcpclients -eq 1 || { status=1; echo "I:failed"; }
 
-n=`expr $n + 1`
-echo_i "reset and check that records are correctly cached initially ($n)"
+echo "I:reset and check that records are correctly cached initially"
 ret=0
 load_cache
 dump_cache
-nrecords=`grep flushtest.example ns2/named_dump.db.$n | grep -v '^;' | wc -l`
-[ $nrecords -eq 20 ] || ret=1
-if [ $ret != 0 ]; then echo_i "failed"; fi
+nrecords=`grep flushtest.example ns2/named_dump.db | grep -v '^;' | egrep '(TXT|ANY)'|  wc -l`
+[ $nrecords -eq 17 ] || { ret=1; echo "I: found $nrecords records expected 17"; }
+if [ $ret != 0 ]; then echo "I:failed"; fi
 status=`expr $status + $ret`
 
-n=`expr $n + 1`
-echo_i "check flushing of the full cache ($n)"
+echo "I:check flushing of the full cache"
 ret=0
 clear_cache
 dump_cache
-nrecords=`grep flushtest.example ns2/named_dump.db.$n | grep -v '^;' | wc -l`
+nrecords=`grep flushtest.example ns2/named_dump.db | grep -v '^;' | wc -l`
 [ $nrecords -eq 0 ] || ret=1
-if [ $ret != 0 ]; then echo_i "failed"; fi
+if [ $ret != 0 ]; then echo "I:failed"; fi
 status=`expr $status + $ret`
 
-n=`expr $n + 1`
-echo_i "check flushing of individual nodes (interior node) ($n)"
+echo "I:check flushing of individual nodes (interior node)"
 ret=0
 clear_cache
 load_cache
@@ -127,21 +121,19 @@ load_cache
 in_cache txt top1.flushtest.example || ret=1
 $RNDC $RNDCOPTS flushname top1.flushtest.example
 in_cache txt top1.flushtest.example && ret=1
-if [ $ret != 0 ]; then echo_i "failed"; fi
+if [ $ret != 0 ]; then echo "I:failed"; fi
 status=`expr $status + $ret`
 
-n=`expr $n + 1`
-echo_i "check flushing of individual nodes (leaf node, under the interior node) ($n)"
+echo "I:check flushing of individual nodes (leaf node, under the interior node)"
 ret=0
 # leaf node, under the interior node (should still exist)
 in_cache txt third2.second1.top1.flushtest.example || ret=1
 $RNDC $RNDCOPTS flushname third2.second1.top1.flushtest.example
 in_cache txt third2.second1.top1.flushtest.example && ret=1
-if [ $ret != 0 ]; then echo_i "failed"; fi
+if [ $ret != 0 ]; then echo "I:failed"; fi
 status=`expr $status + $ret`
 
-n=`expr $n + 1`
-echo_i "check flushing of individual nodes (another leaf node, with both positive and negative cache entries) ($n)"
+echo "I:check flushing of individual nodes (another leaf node, with both positive and negative cache entries)"
 ret=0
 # another leaf node, with both positive and negative cache entries
 in_cache a third1.second1.top1.flushtest.example || ret=1
@@ -149,18 +141,16 @@ in_cache txt third1.second1.top1.flushtest.example || ret=1
 $RNDC $RNDCOPTS flushname third1.second1.top1.flushtest.example
 in_cache a third1.second1.top1.flushtest.example && ret=1
 in_cache txt third1.second1.top1.flushtest.example && ret=1
-if [ $ret != 0 ]; then echo_i "failed"; fi
+if [ $ret != 0 ]; then echo "I:failed"; fi
 status=`expr $status + $ret`
 
-n=`expr $n + 1`
-echo_i "check flushing a nonexistent name ($n)"
+echo "I:check flushing a nonexistent name"
 ret=0
 $RNDC $RNDCOPTS flushname fake.flushtest.example || ret=1
-if [ $ret != 0 ]; then echo_i "failed"; fi
+if [ $ret != 0 ]; then echo "I:failed"; fi
 status=`expr $status + $ret`
 
-n=`expr $n + 1`
-echo_i "check flushing of namespaces ($n)"
+echo "I:check flushing of namespaces"
 ret=0
 clear_cache
 load_cache
@@ -183,42 +173,40 @@ $RNDC $RNDCOPTS flushtree top2.flushtest.example
 in_cache txt second1.top2.flushtest.example && ret=1
 in_cache txt second2.top2.flushtest.example && ret=1
 in_cache txt second3.top2.flushtest.example && ret=1
-if [ $ret != 0 ]; then echo_i "failed"; fi
+if [ $ret != 0 ]; then echo "I:failed"; fi
 status=`expr $status + $ret`
 
-n=`expr $n + 1`
-echo_i "check flushing a nonexistent namespace ($n)"
+echo "I:check flushing a nonexistent namespace"
 ret=0
 $RNDC $RNDCOPTS flushtree fake.flushtest.example || ret=1
-if [ $ret != 0 ]; then echo_i "failed"; fi
+if [ $ret != 0 ]; then echo "I:failed"; fi
 status=`expr $status + $ret`
 
-n=`expr $n + 1`
-echo_i "check the number of cached records remaining ($n)"
+echo "I:check the number of cached records remaining"
 ret=0
 dump_cache
-nrecords=`grep flushtest.example ns2/named_dump.db.$n | grep -v '^;' | wc -l`
-[ $nrecords -eq 19 ] || ret=1
-if [ $ret != 0 ]; then echo_i "failed"; fi
+nrecords=`grep flushtest.example ns2/named_dump.db | grep -v '^;' | egrep '(TXT|ANY)' |  wc -l`
+[ $nrecords -eq 17 ] || { ret=1; echo "I: found $nrecords records expected 17"; }
+if [ $ret != 0 ]; then echo "I:failed"; fi
 status=`expr $status + $ret`
 
-n=`expr $n + 1`
-echo_i "check the check that flushname of a partial match works ($n)"
-ret=0
-in_cache txt second2.top1.flushtest.example || ret=1
-$RNDC $RNDCOPTS flushtree example
-in_cache txt second2.top1.flushtest.example && ret=1
-if [ $ret != 0 ]; then echo_i "failed"; fi
-status=`expr $status + $ret`
-
-n=`expr $n + 1`
-echo_i "check the number of cached records remaining ($n)"
+echo "I:check flushtree clears adb correctly"
 ret=0
+load_cache
 dump_cache
-nrecords=`grep flushtest.example ns2/named_dump.db.$n | grep -v '^;' | egrep '(TXT|ANY)' |  wc -l`
-[ $nrecords -eq 1 ] || { ret=1; echo_i "found $nrecords records expected 1"; }
-if [ $ret != 0 ]; then echo_i "failed"; fi
+awk '/plain success\/timeout/ {getline; getline; if ($2 == "ns.flushtest.example") exit(0); exit(1); }' ns2/named_dump.db || ret=1
+$RNDC $RNDCOPTS flushtree flushtest.example || ret=1
+dump_cache
+awk '/plain success\/timeout/ {getline; getline; if ($2 == "ns.flushtest.example") exit(1); exit(0); }' ns2/named_dump.db || ret=1
+if [ $ret != 0 ]; then echo "I:failed"; fi
 status=`expr $status + $ret`
 
-echo_i "exit status: $status"
-[ $status -eq 0 ] || exit 1
+echo "I:check expire option returned from slave zone"
+ret=0
+$DIG @10.53.0.2 -p 5300 +expire soa expire-test > dig.out.expire
+grep EXPIRE: dig.out.expire > /dev/null || ret=1
+if [ $ret != 0 ]; then echo "I:failed"; fi
+status=`expr $status + $ret`
+
+echo "I:exit status: $status"
+exit $status
