@@ -1,6 +1,7 @@
 #!/bin/sh
 #
-# Copyright (C) Internet Systems Consortium, Inc. ("ISC")
+# Copyright (C) 2004, 2007, 2011, 2012, 2014  Internet Systems Consortium, Inc. ("ISC")
+# Copyright (C) 2001  Internet Software Consortium.
 #
 # Permission to use, copy, modify, and/or distribute this software for any
 # purpose with or without fee is hereby granted, provided that the above
@@ -14,6 +15,9 @@
 # OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 # PERFORMANCE OF THIS SOFTWARE.
 
+# $Id: tests.sh,v 1.11 2012/02/22 14:22:54 marka Exp $
+
+
 # WARNING: The test labelled "testing request-ixfr option in view vs zone"
 #          is fragile because it depends upon counting instances of records
 #          in the log file - need a better approach <sdm> - until then,
@@ -24,11 +28,12 @@ SYSTEMTESTTOP=..
 
 status=0
 
-DIGOPTS="+tcp +noadd +nosea +nostat +noquest +nocomm +nocmd -p ${PORT}"
-SENDCMD="$PERL ../send.pl 10.53.0.2 ${EXTRAPORT1}"
-RNDCCMD="$RNDC -p ${CONTROLPORT} -c ../common/rndc.conf -s"
+DIGOPTS="+tcp +noadd +nosea +nostat +noquest +nocomm +nocmd"
+DIGCMD="$DIG $DIGOPTS @10.53.0.1 -p 5300"
+SENDCMD="$PERL ../send.pl 10.53.0.2 5301"
+RNDCCMD="$RNDC -s 10.53.0.1 -p 9953 -c ../common/rndc.conf"
 
-echo_i "testing initial AXFR"
+echo "I:testing initial AXFR"
 
 $SENDCMD <<EOF
 /SOA/
@@ -58,21 +63,21 @@ zone "nil" {
 };
 EOF
 
-$RNDCCMD 10.53.0.1 reload | sed 's/^/ns1 /' | cat_i
+$RNDCCMD reload
 
 for i in 0 1 2 3 4 5 6 7 8 9
 do
-	$DIG $DIGOPTS @10.53.0.1 nil. SOA > dig.out
+	$DIGCMD nil. SOA > dig.out
 	grep "SOA" dig.out > /dev/null && break
 	sleep 1
 done
 
-$DIG $DIGOPTS @10.53.0.1 nil. TXT | grep 'initial AXFR' >/dev/null || {
-    echo_i "failed"
+$DIGCMD nil. TXT | grep 'initial AXFR' >/dev/null || {
+    echo "I:failed"
     status=1
 }
 
-echo_i "testing successful IXFR"
+echo "I:testing successful IXFR"
 
 # We change the IP address of a.nil., and the TXT record at the apex.
 # Then we do a SOA-only update.
@@ -95,16 +100,16 @@ EOF
 
 sleep 1
 
-$RNDCCMD 10.53.0.1 refresh nil
+$RNDCCMD refresh nil
 
 sleep 2
 
-$DIG $DIGOPTS @10.53.0.1 nil. TXT | grep 'successful IXFR' >/dev/null || {
-    echo_i "failed"
+$DIGCMD nil. TXT | grep 'successful IXFR' >/dev/null || {
+    echo "I:failed"
     status=1
 }
 
-echo_i "testing AXFR fallback after IXFR failure"
+echo "I:testing AXFR fallback after IXFR failure"
 
 # Provide a broken IXFR response and a working fallback AXFR response
 
@@ -129,61 +134,38 @@ EOF
 
 sleep 1
 
-$RNDCCMD 10.53.0.1 refresh nil
+$RNDCCMD refresh nil
 
 sleep 2
 
-$DIG $DIGOPTS @10.53.0.1 nil. TXT | grep 'fallback AXFR' >/dev/null || {
-    echo_i "failed"
+$DIGCMD nil. TXT | grep 'fallback AXFR' >/dev/null || {
+    echo "I:failed"
     status=1
 }
 
-echo_i "testing ixfr-from-differences option"
-# ns3 is master; ns4 is slave
+echo "I:testing ixfr-from-differences option"
+# ns3 is master; ns4 is slave 
 $CHECKZONE test. ns3/mytest.db > /dev/null 2>&1
 if [ $? -ne 0 ]
 then
-    echo_i "named-checkzone returned failure on ns3/mytest.db"
+    echo "I:named-checkzone returned failure on ns3/mytest.db"
 fi
 # modify the master
-#echo_i "digging against master: "
-#$DIG $DIGOPTS @10.53.0.3 a host1.test.
-#echo_i "digging against slave: "
-#$DIG $DIGOPTS @10.53.0.4 a host1.test.
+#echo "I: digging against master: "
+#$DIG $DIGOPTS @10.53.0.3 -p 5300 a host1.test.
+#echo "I: digging against slave: "
+#$DIG $DIGOPTS @10.53.0.4 -p 5300 a host1.test.
 
-# wait for slave to be stable
-for i in 0 1 2 3 4 5 6 7 8 9
-do
-	$DIG $DIGOPTS +tcp @10.53.0.4 SOA test > dig.out
-	grep -i "hostmaster\.test\..1" dig.out > /dev/null && break
-	sleep 1
-done
-
-# modify the master
 cp ns3/mytest1.db ns3/mytest.db
-$RNDCCMD 10.53.0.3 reload | sed 's/^/ns3 /' | cat_i
+$RNDC -s 10.53.0.3 -p 9953 -c ../common/rndc.conf reload
 
-#wait for master to reload load
 for i in 0 1 2 3 4 5 6 7 8 9
 do
-	$DIG $DIGOPTS +tcp @10.53.0.3 SOA test > dig.out
+	$DIG +tcp -p 5300 @10.53.0.4 SOA test > dig.out
 	grep -i "hostmaster\.test\..2" dig.out > /dev/null && break
 	sleep 1
 done
 
-#wait for slave to transfer zone
-for i in 0 1 2 3 4 5 6 7 8 9 10 11 12 13 14
-do
-	$DIG $DIGOPTS +tcp @10.53.0.4 SOA test > dig.out
-	grep -i "hostmaster\.test\..2" dig.out > /dev/null && break
-
-	# re-notify if we've been waiting a long time
-	if [ $i -ge 5 ]; then
-	    $RNDCCMD 10.53.0.3 notify test | set 's/^/ns3 /' | cat_i
-	fi
-	sleep 1
-done
-
 # slave should have gotten notify and updated
 
 for i in 0 1 2 3 4 5 6 7 8 9
@@ -194,42 +176,28 @@ do
 done
 if [ $INCR -ne 1 ]
 then
-    echo_i "failed to get incremental response"
+    echo "I:failed to get incremental response"
     status=1
 fi
 
-echo_i "testing request-ixfr option in view vs zone"
+echo "I:testing request-ixfr option in view vs zone"
 # There's a view with 2 zones. In the view, "request-ixfr yes"
 # but in the zone "sub.test", request-ixfr no"
 # we want to make sure that a change to sub.test results in AXFR, while
 # changes to test. result in IXFR
 
-echo_i " this result should be AXFR"
+echo "I: this result should be AXFR"
 cp ns3/subtest1.db ns3/subtest.db # change to sub.test zone, should be AXFR
-$RNDCCMD 10.53.0.3 reload | sed 's/^/ns3 /' | cat_i
+$RNDC -s 10.53.0.3 -p 9953 -c ../common/rndc.conf reload
 
-#wait for master to reload zone
 for i in 0 1 2 3 4 5 6 7 8 9
 do
-	$DIG $DIGOPTS +tcp @10.53.0.3 SOA sub.test > dig.out
+	$DIG +tcp -p 5300 @10.53.0.4 SOA sub.test > dig.out
 	grep -i "hostmaster\.test\..3" dig.out > /dev/null && break
 	sleep 1
 done
 
-#wait for slave to transfer zone
-for i in 0 1 2 3 4 5 6 7 8 9 10 11 12 13 14
-do
-	$DIG $DIGOPTS +tcp @10.53.0.4 SOA sub.test > dig.out
-	grep -i "hostmaster\.test\..3" dig.out > /dev/null && break
-
-	# re-notify if we've been waiting a long time
-	if [ $i -ge 5 ]; then
-	    $RNDCCMD 10.53.0.3 notify sub.test | set 's/^/ns3 /' | cat_i
-	fi
-	sleep 1
-done
-
-echo_i " this result should be AXFR"
+echo "I: this result should be AXFR"
 for i in 0 1 2 3 4 5 6 7 8 9
 do
 	NONINCR=`grep 'sub\.test/IN/primary' ns4/named.run|grep "got nonincremental" | wc -l`
@@ -238,37 +206,23 @@ do
 done
 if [ $NONINCR -ne 2 ]
 then
-    echo_i "failed to get nonincremental response in 2nd AXFR test"
+    echo "I:failed to get nonincremental response in 2nd AXFR test"
     status=1
 else
-    echo_i "  success: AXFR it was"
+    echo "I:  success: AXFR it was"
 fi
 
-echo_i " this result should be IXFR"
+echo "I: this result should be IXFR"
 cp ns3/mytest2.db ns3/mytest.db # change to test zone, should be IXFR
-$RNDCCMD 10.53.0.3 reload | sed 's/^/ns3 /' | cat_i
+$RNDC -s 10.53.0.3 -p 9953 -c ../common/rndc.conf reload
 
-# wait for master to reload zone
 for i in 0 1 2 3 4 5 6 7 8 9
 do
-	$DIG +tcp -p 5300 @10.53.0.3 SOA test > dig.out
+	$DIG +tcp -p 5300 @10.53.0.4 SOA test > dig.out
 	grep -i "hostmaster\.test\..4" dig.out > /dev/null && break
 	sleep 1
 done
 
-# wait for slave to transfer zone
-for i in 0 1 2 3 4 5 6 7 8 9 10 11 12 13 14
-do
-	$DIG $DIGOPTS +tcp @10.53.0.4 SOA test > dig.out
-	grep -i "hostmaster\.test\..4" dig.out > /dev/null && break
-
-	# re-notify if we've been waiting a long time
-	if [ $i -ge 5 ]; then
-	    $RNDCCMD 10.53.0.3 notify test | set 's/^/ns3 /' | cat_i
-	fi
-	sleep 1
-done
-
 for i in 0 1 2 3 4 5 6 7 8 9
 do
 	INCR=`grep "test/IN/primary" ns4/named.run|grep "got incremental"|wc -l`
@@ -277,39 +231,23 @@ do
 done
 if [ $INCR -ne 2 ]
 then
-    echo_i "failed to get incremental response in 2nd IXFR test"
+    echo "I:failed to get incremental response in 2nd IXFR test"
     status=1
 else
-    echo_i "  success: IXFR it was"
+    echo "I:  success: IXFR it was"
 fi
 
-echo_i "testing DiG's handling of a multi message AXFR style IXFR response"
+echo "I:testing DiG's handling of a multi message AXFR style IXFR response" 
 (
 (sleep 10 && kill $$) 2>/dev/null &
 sub=$!
-$DIG -p ${PORT} ixfr=0 large @10.53.0.3 > dig.out
+$DIG ixfr=0 large -p 5300 @10.53.0.3 > dig.out
 kill $sub
 )
 lines=`grep hostmaster.large dig.out | wc -l`
-test ${lines:-0} -eq 2 || { echo_i "failed"; status=1; }
+test ${lines:-0} -eq 2 || { echo "I:failed"; status=1; }
 messages=`sed -n 's/^;;.*messages \([0-9]*\),.*/\1/p' dig.out`
-test ${messages:-0} -gt 1 || { echo_i "failed"; status=1; }
+test ${messages:-0} -gt 1 || { echo "I:failed"; status=1; }
 
-echo_i "test 'dig +notcp ixfr=<value>' vs 'dig ixfr=<value> +notcp' vs 'dig ixfr=<value>'"
-ret=0
-# Should be "switch to TCP" response
-$DIG $DIGOPTS +notcp ixfr=1 test @10.53.0.4 > dig.out1 || ret=1
-$DIG $DIGOPTS ixfr=1 +notcp test @10.53.0.4 > dig.out2 || ret=1
-digcomp dig.out1 dig.out2 || ret=1
-awk '$4 == "SOA" { soacnt++} END {if (soacnt == 1) exit(0); else exit(1);}' dig.out1 || ret=1
-awk '$4 == "SOA" { if ($7 == 4) exit(0); else exit(1);}' dig.out1 || ret=1
-# Should be incremental transfer.
-$DIG $DIGOPTS ixfr=1 test @10.53.0.4 > dig.out3 || ret=1
-awk '$4 == "SOA" { soacnt++} END { if (soacnt == 6) exit(0); else exit(1);}' dig.out3 || ret=1
-if [ ${ret} != 0 ]; then
-	echo_i "failed";
-	status=1;
-fi
-
-echo_i "exit status: $status"
-[ $status -eq 0 ] || exit 1
+echo "I:exit status: $status"
+exit $status
