@@ -1,28 +1,31 @@
 /*
- * Copyright (C) Internet Systems Consortium, Inc. ("ISC")
+ * Copyright (C) 2004-2010, 2012-2014  Internet Systems Consortium, Inc. ("ISC")
+ * Copyright (C) 1999-2003  Internet Software Consortium.
  *
- * SPDX-License-Identifier: MPL-2.0
+ * Permission to use, copy, modify, and/or distribute this software for any
+ * purpose with or without fee is hereby granted, provided that the above
+ * copyright notice and this permission notice appear in all copies.
  *
- * This Source Code Form is subject to the terms of the Mozilla Public
- * License, v. 2.0. If a copy of the MPL was not distributed with this
- * file, you can obtain one at https://mozilla.org/MPL/2.0/.
- *
- * See the COPYRIGHT file distributed with this work for additional
- * information regarding copyright ownership.
+ * THE SOFTWARE IS PROVIDED "AS IS" AND ISC DISCLAIMS ALL WARRANTIES WITH
+ * REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
+ * AND FITNESS.  IN NO EVENT SHALL ISC BE LIABLE FOR ANY SPECIAL, DIRECT,
+ * INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
+ * LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE
+ * OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
+ * PERFORMANCE OF THIS SOFTWARE.
  */
 
-#pragma once
+/* $Id$ */
+
+#ifndef DNS_MESSAGE_H
+#define DNS_MESSAGE_H 1
 
 /***
  ***	Imports
  ***/
 
-#include <inttypes.h>
-#include <stdbool.h>
-
 #include <isc/lang.h>
 #include <isc/magic.h>
-#include <isc/refcount.h>
 
 #include <dns/compress.h>
 #include <dns/masterdump.h>
@@ -30,14 +33,12 @@
 
 #include <dst/dst.h>
 
-/* Add -DDNS_MESSAGE_TRACE=1 to CFLAGS for detailed reference tracing */
-
 /*! \file dns/message.h
  * \brief Message Handling Module
  *
  * How this beast works:
  *
- * When a dns message is received in a buffer, dns_message_parse() is called
+ * When a dns message is received in a buffer, dns_message_fromwire() is called
  * on the memory region.  Various items are checked including the format
  * of the message (if counts are right, if counts consume the entire sections,
  * and if sections consume the entire message) and known pseudo-RRs in the
@@ -78,7 +79,8 @@
  * \code
  *	buffer = isc_buffer_allocate(mctx, 512);
  *	name = NULL;
- *	result = dns_message_gettempname(message, &name);
+ *	name = dns_message_gettempname(message, &name);
+ *	dns_name_init(name, NULL);
  *	result = dns_name_fromtext(name, &source, dns_rootname, 0, buffer);
  *	dns_message_takebuffer(message, &buffer);
  * \endcode
@@ -90,265 +92,179 @@
  * section, move rdata from one section to another, remove rdata, etc.
  */
 
-#define DNS_MESSAGEFLAG_QR 0x8000U
-#define DNS_MESSAGEFLAG_AA 0x0400U
-#define DNS_MESSAGEFLAG_TC 0x0200U
-#define DNS_MESSAGEFLAG_RD 0x0100U
-#define DNS_MESSAGEFLAG_RA 0x0080U
-#define DNS_MESSAGEFLAG_AD 0x0020U
-#define DNS_MESSAGEFLAG_CD 0x0010U
+#define DNS_MESSAGEFLAG_QR		0x8000U
+#define DNS_MESSAGEFLAG_AA		0x0400U
+#define DNS_MESSAGEFLAG_TC		0x0200U
+#define DNS_MESSAGEFLAG_RD		0x0100U
+#define DNS_MESSAGEFLAG_RA		0x0080U
+#define DNS_MESSAGEFLAG_AD		0x0020U
+#define DNS_MESSAGEFLAG_CD		0x0010U
 
 /*%< EDNS0 extended message flags */
-#define DNS_MESSAGEEXTFLAG_DO 0x8000U
+#define DNS_MESSAGEEXTFLAG_DO		0x8000U
 
 /*%< EDNS0 extended OPT codes */
-#define DNS_OPT_LLQ	      1	 /*%< LLQ opt code */
-#define DNS_OPT_NSID	      3	 /*%< NSID opt code */
-#define DNS_OPT_CLIENT_SUBNET 8	 /*%< client subnet opt code */
-#define DNS_OPT_EXPIRE	      9	 /*%< EXPIRE opt code */
-#define DNS_OPT_COOKIE	      10 /*%< COOKIE opt code */
-#define DNS_OPT_TCP_KEEPALIVE 11 /*%< TCP keepalive opt code */
-#define DNS_OPT_PAD	      12 /*%< PAD opt code */
-#define DNS_OPT_KEY_TAG	      14 /*%< Key tag opt code */
-#define DNS_OPT_EDE	      15 /*%< Extended DNS Error opt code */
-#define DNS_OPT_CLIENT_TAG    16 /*%< Client tag opt code */
-#define DNS_OPT_SERVER_TAG    17 /*%< Server tag opt code */
-
+#define DNS_OPT_NSID		0x0003		/*%< NSID opt code */
+#define DNS_OPT_CLIENT_SUBNET	0x0008		/*%< client subnet opt code */
 /*%< Experimental options [65001...65534] as per RFC6891 */
+#define DNS_OPT_SIT		65001		/*%< SIT opt code */
+#define DNS_OPT_EXPIRE		65002		/*%< EXPIRE opt code */
 
-/*%<
- * The maximum number of EDNS options we allow to set. Reserve space for the
- * options we know about. Extended DNS Errors may occur multiple times, but we
- * will set only one per message (for now).
- */
-#define DNS_EDNSOPTIONS 8
+/*%< The number of EDNS options we know about. */
+#define DNS_EDNSOPTIONS	4
 
-/*%< EDNS0 extended DNS errors */
-#define DNS_EDE_OTHER		     0	/*%< Other Error */
-#define DNS_EDE_DNSKEYALG	     1	/*%< Unsupported DNSKEY Algorithm */
-#define DNS_EDE_DSDIGESTTYPE	     2	/*%< Unsupported DS Digest Type */
-#define DNS_EDE_STALEANSWER	     3	/*%< Stale Answer */
-#define DNS_EDE_FORGEDANSWER	     4	/*%< Forged Answer */
-#define DNS_EDE_DNSSECINDETERMINATE  5	/*%< DNSSEC Indeterminate */
-#define DNS_EDE_DNSSECBOGUS	     6	/*%< DNSSEC Bogus */
-#define DNS_EDE_SIGNATUREEXPIRED     7	/*%< Signature Expired */
-#define DNS_EDE_SIGNATURENOTYETVALID 8	/*%< Signature Not Yet Valid */
-#define DNS_EDE_DNSKEYMISSING	     9	/*%< DNSKEY Missing */
-#define DNS_EDE_RRSIGSMISSING	     10 /*%< RRSIGs Missing */
-#define DNS_EDE_NOZONEKEYBITSET	     11 /*%< No Zone Key Bit Set */
-#define DNS_EDE_NSECMISSING	     12 /*%< NSEC Missing */
-#define DNS_EDE_CACHEDERROR	     13 /*%< Cached Error */
-#define DNS_EDE_NOTREADY	     14 /*%< Not Ready */
-#define DNS_EDE_BLOCKED		     15 /*%< Blocked */
-#define DNS_EDE_CENSORED	     16 /*%< Censored */
-#define DNS_EDE_FILTERED	     17 /*%< Filtered */
-#define DNS_EDE_PROHIBITED	     18 /*%< Prohibited */
-#define DNS_EDE_STALENXANSWER	     19 /*%< Stale NXDomain Answer */
-#define DNS_EDE_NOTAUTH		     20 /*%< Not Authoritative */
-#define DNS_EDE_NOTSUPPORTED	     21 /*%< Not Supported */
-#define DNS_EDE_NOREACHABLEAUTH	     22 /*%< No Reachable Authority */
-#define DNS_EDE_NETWORKERROR	     23 /*%< Network Error */
-#define DNS_EDE_INVALIDDATA	     24 /*%< Invalid Data */
-
-/*
- * From RFC 8914:
- * Because long EXTRA-TEXT fields may trigger truncation (which is undesirable
- * given the supplemental nature of EDE), implementers and operators creating
- * EDE options SHOULD avoid lengthy EXTRA-TEXT contents.
- *
- * Following this advice we limit the EXTRA-TEXT length to 64 characters.
- */
-#define DNS_EDE_EXTRATEXT_LEN 64
-
-#define DNS_MESSAGE_REPLYPRESERVE	 (DNS_MESSAGEFLAG_RD | DNS_MESSAGEFLAG_CD)
+#define DNS_MESSAGE_REPLYPRESERVE	(DNS_MESSAGEFLAG_RD|DNS_MESSAGEFLAG_CD)
 #define DNS_MESSAGEEXTFLAG_REPLYPRESERVE (DNS_MESSAGEEXTFLAG_DO)
 
-#define DNS_MESSAGE_HEADERLEN 12 /*%< 6 uint16_t's */
+#define DNS_MESSAGE_HEADERLEN		12 /*%< 6 isc_uint16_t's */
 
-#define DNS_MESSAGE_MAGIC      ISC_MAGIC('M', 'S', 'G', '@')
-#define DNS_MESSAGE_VALID(msg) ISC_MAGIC_VALID(msg, DNS_MESSAGE_MAGIC)
+#define DNS_MESSAGE_MAGIC		ISC_MAGIC('M','S','G','@')
+#define DNS_MESSAGE_VALID(msg)		ISC_MAGIC_VALID(msg, DNS_MESSAGE_MAGIC)
 
 /*
  * Ordering here matters.  DNS_SECTION_ANY must be the lowest and negative,
  * and DNS_SECTION_MAX must be one greater than the last used section.
  */
 typedef int dns_section_t;
-#define DNS_SECTION_ANY	       (-1)
-#define DNS_SECTION_QUESTION   0
-#define DNS_SECTION_ANSWER     1
-#define DNS_SECTION_AUTHORITY  2
-#define DNS_SECTION_ADDITIONAL 3
-#define DNS_SECTION_MAX	       4
+#define DNS_SECTION_ANY			(-1)
+#define DNS_SECTION_QUESTION		0
+#define DNS_SECTION_ANSWER		1
+#define DNS_SECTION_AUTHORITY		2
+#define DNS_SECTION_ADDITIONAL		3
+#define DNS_SECTION_MAX			4
 
 typedef int dns_pseudosection_t;
-#define DNS_PSEUDOSECTION_ANY  (-1)
-#define DNS_PSEUDOSECTION_OPT  0
-#define DNS_PSEUDOSECTION_TSIG 1
-#define DNS_PSEUDOSECTION_SIG0 2
-#define DNS_PSEUDOSECTION_MAX  3
+#define DNS_PSEUDOSECTION_ANY		(-1)
+#define DNS_PSEUDOSECTION_OPT           0
+#define DNS_PSEUDOSECTION_TSIG          1
+#define DNS_PSEUDOSECTION_SIG0          2
+#define DNS_PSEUDOSECTION_MAX           3
 
 typedef int dns_messagetextflag_t;
-#define DNS_MESSAGETEXTFLAG_NOCOMMENTS 0x0001
-#define DNS_MESSAGETEXTFLAG_NOHEADERS  0x0002
-#define DNS_MESSAGETEXTFLAG_ONESOA     0x0004
-#define DNS_MESSAGETEXTFLAG_OMITSOA    0x0008
+#define DNS_MESSAGETEXTFLAG_NOCOMMENTS	0x0001
+#define DNS_MESSAGETEXTFLAG_NOHEADERS	0x0002
+#define DNS_MESSAGETEXTFLAG_ONESOA	0x0004
+#define DNS_MESSAGETEXTFLAG_OMITSOA	0x0008
+#define DNS_MESSAGETEXTFLAG_COMMENTDATA	0x0010
 
 /*
  * Dynamic update names for these sections.
  */
-#define DNS_SECTION_ZONE	 DNS_SECTION_QUESTION
-#define DNS_SECTION_PREREQUISITE DNS_SECTION_ANSWER
-#define DNS_SECTION_UPDATE	 DNS_SECTION_AUTHORITY
+#define DNS_SECTION_ZONE		DNS_SECTION_QUESTION
+#define DNS_SECTION_PREREQUISITE	DNS_SECTION_ANSWER
+#define DNS_SECTION_UPDATE		DNS_SECTION_AUTHORITY
 
 /*
  * These tell the message library how the created dns_message_t will be used.
  */
-typedef enum dns_message_intent {
-	DNS_MESSAGE_INTENTUNKNOWN = 0, /*%< internal use only */
-	DNS_MESSAGE_INTENTPARSE = 1,   /*%< parsing messages */
-	DNS_MESSAGE_INTENTRENDER = 2,  /*%< rendering */
-} dns_message_intent_t;
+#define DNS_MESSAGE_INTENTUNKNOWN	0 /*%< internal use only */
+#define DNS_MESSAGE_INTENTPARSE		1 /*%< parsing messages */
+#define DNS_MESSAGE_INTENTRENDER	2 /*%< rendering */
 
 /*
  * Control behavior of parsing
  */
-#define DNS_MESSAGEPARSE_PRESERVEORDER 0x0001 /*%< preserve rdata order */
-#define DNS_MESSAGEPARSE_BESTEFFORT       \
-	0x0002 /*%< return a message if a \
-		* recoverable parse error \
-		* occurs */
-#define DNS_MESSAGEPARSE_CLONEBUFFER   \
-	0x0004 /*%< save a copy of the \
-		* source buffer */
-#define DNS_MESSAGEPARSE_IGNORETRUNCATION \
-	0x0008 /*%< truncation errors are \
-		* not fatal. */
+#define DNS_MESSAGEPARSE_PRESERVEORDER	0x0001	/*%< preserve rdata order */
+#define DNS_MESSAGEPARSE_BESTEFFORT	0x0002	/*%< return a message if a
+						   recoverable parse error
+						   occurs */
+#define DNS_MESSAGEPARSE_CLONEBUFFER	0x0004	/*%< save a copy of the
+						   source buffer */
+#define DNS_MESSAGEPARSE_IGNORETRUNCATION 0x0008 /*%< truncation errors are
+						  * not fatal. */
 
 /*
  * Control behavior of rendering
  */
-#define DNS_MESSAGERENDER_ORDERED    0x0001 /*%< don't change order */
-#define DNS_MESSAGERENDER_PARTIAL    0x0002 /*%< allow a partial rdataset */
-#define DNS_MESSAGERENDER_OMITDNSSEC 0x0004 /*%< omit DNSSEC records */
-#define DNS_MESSAGERENDER_PREFER_A      \
-	0x0008 /*%< prefer A records in \
-		*   additional section. */
-#define DNS_MESSAGERENDER_PREFER_AAAA      \
-	0x0010 /*%< prefer AAAA records in \
-		*   additional section. */
-/* Obsolete: DNS_MESSAGERENDER_FILTER_AAAA	0x0020	*/
+#define DNS_MESSAGERENDER_ORDERED	0x0001	/*%< don't change order */
+#define DNS_MESSAGERENDER_PARTIAL	0x0002	/*%< allow a partial rdataset */
+#define DNS_MESSAGERENDER_OMITDNSSEC	0x0004	/*%< omit DNSSEC records */
+#define DNS_MESSAGERENDER_PREFER_A	0x0008	/*%< prefer A records in
+						      additional section. */
+#define DNS_MESSAGERENDER_PREFER_AAAA	0x0010	/*%< prefer AAAA records in
+						  additional section. */
+#ifdef ALLOW_FILTER_AAAA
+#define DNS_MESSAGERENDER_FILTER_AAAA	0x0020	/*%< filter AAAA records */
+#endif
 
 typedef struct dns_msgblock dns_msgblock_t;
 
-struct dns_sortlist_arg {
-	dns_aclenv_t	       *env;
-	dns_acl_t	       *acl;
-	const dns_aclelement_t *element;
-};
-
-typedef struct dns_minttl {
-	bool	  is_set;
-	dns_ttl_t ttl;
-} dns_minttl_t;
-
 struct dns_message {
 	/* public from here down */
-	unsigned int   magic;
-	isc_refcount_t references;
+	unsigned int			magic;
 
-	dns_messageid_t	 id;
-	unsigned int	 flags;
-	dns_rcode_t	 rcode;
-	dns_opcode_t	 opcode;
-	dns_rdataclass_t rdclass;
+	dns_messageid_t			id;
+	unsigned int			flags;
+	dns_rcode_t			rcode;
+	unsigned int			opcode;
+	dns_rdataclass_t		rdclass;
 
 	/* 4 real, 1 pseudo */
-	unsigned int counts[DNS_SECTION_MAX];
+	unsigned int			counts[DNS_SECTION_MAX];
 
 	/* private from here down */
-	dns_namelist_t	sections[DNS_SECTION_MAX];
-	dns_name_t     *cursors[DNS_SECTION_MAX];
-	dns_rdataset_t *opt;
-	dns_rdataset_t *sig0;
-	dns_rdataset_t *tsig;
+	dns_namelist_t			sections[DNS_SECTION_MAX];
+	dns_name_t		       *cursors[DNS_SECTION_MAX];
+	dns_rdataset_t		       *opt;
+	dns_rdataset_t		       *sig0;
+	dns_rdataset_t		       *tsig;
 
-	int state;
-	unsigned int			      : 0; /* bits */
-	dns_message_intent_t from_to_wire     : 2; /* 2 */
-	unsigned int	     header_ok	      : 1; /* 3 */
-	unsigned int	     question_ok      : 1; /* 4 */
-	unsigned int	     tcp_continuation : 1; /* 5 */
-	unsigned int	     verified_sig     : 1; /* 6 */
-	unsigned int	     verify_attempted : 1; /* 7 */
-	unsigned int	     free_query	      : 1; /* 8 */
-	unsigned int	     free_saved	      : 1; /* 9 */
-	unsigned int	     cc_ok	      : 1; /* 10 */
-	unsigned int	     cc_bad	      : 1; /* 11 */
-	unsigned int	     cc_echoed	      : 1; /* 12 */
-	unsigned int	     tkey	      : 1; /* 13 */
-	unsigned int	     rdclass_set      : 1; /* 14 */
-	unsigned int	     fuzzing	      : 1; /* 15 */
-	unsigned int	     free_pools	      : 1; /* 16 */
-	unsigned int			      : 0;
+	int				state;
+	unsigned int			from_to_wire : 2;
+	unsigned int			header_ok : 1;
+	unsigned int			question_ok : 1;
+	unsigned int			tcp_continuation : 1;
+	unsigned int			verified_sig : 1;
+	unsigned int			verify_attempted : 1;
+	unsigned int			free_query : 1;
+	unsigned int			free_saved : 1;
+	unsigned int			sitok : 1;
+	unsigned int			sitbad : 1;
 
-	unsigned int opt_reserved;
-	unsigned int sig_reserved;
-	unsigned int reserved; /* reserved space (render) */
+	unsigned int			opt_reserved;
+	unsigned int			sig_reserved;
+	unsigned int			reserved; /* reserved space (render) */
 
-	uint16_t     padding;
-	unsigned int padding_off;
+	isc_buffer_t		       *buffer;
+	dns_compress_t		       *cctx;
 
-	isc_buffer_t   *buffer;
-	dns_compress_t *cctx;
+	isc_mem_t		       *mctx;
+	isc_mempool_t		       *namepool;
+	isc_mempool_t		       *rdspool;
 
-	isc_mem_t     *mctx;
-	isc_mempool_t *namepool;
-	isc_mempool_t *rdspool;
+	isc_bufferlist_t		scratchpad;
+	isc_bufferlist_t		cleanup;
 
-	isc_bufferlist_t scratchpad;
-	isc_bufferlist_t cleanup;
+	ISC_LIST(dns_msgblock_t)	rdatas;
+	ISC_LIST(dns_msgblock_t)	rdatalists;
+	ISC_LIST(dns_msgblock_t)	offsets;
 
-	ISC_LIST(dns_msgblock_t) rdatas;
-	ISC_LIST(dns_msgblock_t) rdatalists;
-	ISC_LIST(dns_msgblock_t) offsets;
+	ISC_LIST(dns_rdata_t)		freerdata;
+	ISC_LIST(dns_rdatalist_t)	freerdatalist;
 
-	ISC_LIST(dns_rdata_t) freerdata;
-	ISC_LIST(dns_rdatalist_t) freerdatalist;
+	dns_rcode_t			tsigstatus;
+	dns_rcode_t			querytsigstatus;
+	dns_name_t		       *tsigname; /* Owner name of TSIG, if any */
+	dns_rdataset_t		       *querytsig;
+	dns_tsigkey_t		       *tsigkey;
+	dst_context_t		       *tsigctx;
+	int				sigstart;
+	int				timeadjust;
 
-	dns_rcode_t tsigstatus;
-	dns_rcode_t querytsigstatus;
-	dns_name_t *tsigname; /* Owner name of TSIG, if any
-			       * */
-	dns_rdataset_t *querytsig;
-	dns_tsigkey_t  *tsigkey;
-	dst_context_t  *tsigctx;
-	int		sigstart;
-	int		timeadjust;
+	dns_name_t		       *sig0name; /* Owner name of SIG0, if any */
+	dst_key_t		       *sig0key;
+	dns_rcode_t			sig0status;
+	isc_region_t			query;
+	isc_region_t			saved;
 
-	dns_name_t *sig0name; /* Owner name of SIG0, if any
-			       * */
-	dst_key_t   *sig0key;
-	dns_rcode_t  sig0status;
-	isc_region_t query;
-	isc_region_t saved;
-
-	/*
-	 * Time to be used when fuzzing.
-	 */
-	isc_stdtime_t fuzztime;
-
-	dns_rdatasetorderfunc_t order;
-	dns_sortlist_arg_t	order_arg;
-
-	dns_indent_t indent;
-
-	dns_minttl_t minttl[DNS_SECTION_MAX];
+	dns_rdatasetorderfunc_t		order;
+	const void *			order_arg;
 };
 
 struct dns_ednsopt {
-	uint16_t       code;
-	uint16_t       length;
-	unsigned char *value;
+	isc_uint16_t			code;
+	isc_uint16_t			length;
+	unsigned char			*value;
 };
 
 /***
@@ -357,10 +273,9 @@ struct dns_ednsopt {
 
 ISC_LANG_BEGINDECLS
 
-void
-dns_message_create(isc_mem_t *mctx, isc_mempool_t *namepool,
-		   isc_mempool_t *rdspool, dns_message_intent_t intent,
-		   dns_message_t **msgp);
+isc_result_t
+dns_message_create(isc_mem_t *mctx, unsigned int intent, dns_message_t **msgp);
+
 /*%<
  * Create msg structure.
  *
@@ -372,24 +287,25 @@ dns_message_create(isc_mem_t *mctx, isc_mempool_t *namepool,
  *
  *\li	'msgp' be non-null and '*msg' be NULL.
  *
- *\li	'namepool' and 'rdspool' must be either both NULL or both valid
- *	isc_mempool_t
- *
  *\li	'intent' must be one of DNS_MESSAGE_INTENTPARSE or
  *	#DNS_MESSAGE_INTENTRENDER.
  *
  * Ensures:
  *\li	The data in "*msg" is set to indicate an unused and empty msg
  *	structure.
+ *
+ * Returns:
+ *\li	#ISC_R_NOMEMORY		-- out of memory
+ *\li	#ISC_R_SUCCESS		-- success
  */
 
 void
-dns_message_reset(dns_message_t *msg, dns_message_intent_t intent);
+dns_message_reset(dns_message_t *msg, unsigned int intent);
 /*%<
  * Reset a message structure to default state.  All internal lists are freed
  * or reset to a default state as well.  This is simply a more efficient
- * way to call dns_message_detach() (assuming last reference is hold),
- * followed by dns_message_create(), since it avoid many memory allocations.
+ * way to call dns_message_destroy() followed by dns_message_allocate(),
+ * since it avoid many memory allocations.
  *
  * If any data loanouts (buffers, names, rdatas, etc) were requested,
  * the caller must no longer use them after this call.
@@ -403,32 +319,31 @@ dns_message_reset(dns_message_t *msg, dns_message_intent_t intent);
  *\li	'intent' is DNS_MESSAGE_INTENTPARSE or DNS_MESSAGE_INTENTRENDER
  */
 
-#if DNS_MESSAGE_TRACE
-#define dns_message_ref(ptr) dns_message__ref(ptr, __func__, __FILE__, __LINE__)
-#define dns_message_unref(ptr) \
-	dns_message__unref(ptr, __func__, __FILE__, __LINE__)
-#define dns_message_attach(ptr, ptrp) \
-	dns_message__attach(ptr, ptrp, __func__, __FILE__, __LINE__)
-#define dns_message_detach(ptrp) \
-	dns_message__detach(ptrp, __func__, __FILE__, __LINE__)
-ISC_REFCOUNT_TRACE_DECL(dns_message);
-#else
-ISC_REFCOUNT_DECL(dns_message);
-#endif
-/*
- * Reference counting for dns_message
+void
+dns_message_destroy(dns_message_t **msgp);
+/*%<
+ * Destroy all state in the message.
+ *
+ * Requires:
+ *
+ *\li	'msgp' be valid.
+ *
+ * Ensures:
+ *\li	'*msgp' == NULL
  */
 
 isc_result_t
 dns_message_sectiontotext(dns_message_t *msg, dns_section_t section,
 			  const dns_master_style_t *style,
-			  dns_messagetextflag_t flags, isc_buffer_t *target);
+			  dns_messagetextflag_t flags,
+			  isc_buffer_t *target);
 
 isc_result_t
-dns_message_pseudosectiontotext(dns_message_t *msg, dns_pseudosection_t section,
+dns_message_pseudosectiontotext(dns_message_t *msg,
+				dns_pseudosection_t section,
 				const dns_master_style_t *style,
-				dns_messagetextflag_t	  flags,
-				isc_buffer_t		 *target);
+				dns_messagetextflag_t flags,
+				isc_buffer_t *target);
 /*%<
  * Convert section 'section' or 'pseudosection' of message 'msg' to
  * a cleartext representation
@@ -444,7 +359,7 @@ dns_message_pseudosectiontotext(dns_message_t *msg, dns_pseudosection_t section,
  *
  *\li	'target' is a valid buffer.
  *
- *\li	'section' is a named section label.
+ *\li	'section' is a valid section label.
  *
  * Ensures:
  *
@@ -458,38 +373,7 @@ dns_message_pseudosectiontotext(dns_message_t *msg, dns_pseudosection_t section,
  *\li	#ISC_R_NOMORE
  *
  *\li	Note: On error return, *target may be partially filled with data.
- */
-
-isc_result_t
-dns_message_headertotext(dns_message_t *msg, const dns_master_style_t *style,
-			 dns_messagetextflag_t flags, isc_buffer_t *target);
-/*%<
- * Convert the header section of message 'msg' to a cleartext
- * representation. This is called from dns_message_totext().
- *
- * Notes on flags:
- *\li	If #DNS_MESSAGETEXTFLAG_NOHEADERS is set, header lines will be
- * 	suppressed and this function is a no-op.
- *
- * Requires:
- *
- *\li	'msg' is a valid message.
- *
- *\li	'target' is a valid buffer.
- *
- * Ensures:
- *
- *\li	If the result is success:
- *		The used space in 'target' is updated.
- *
- * Returns:
- *
- *\li	#ISC_R_SUCCESS
- *\li	#ISC_R_NOSPACE
- *\li	#ISC_R_NOMORE
- *
- *\li	Note: On error return, *target may be partially filled with data.
- */
+*/
 
 isc_result_t
 dns_message_totext(dns_message_t *msg, const dns_master_style_t *style,
@@ -497,20 +381,21 @@ dns_message_totext(dns_message_t *msg, const dns_master_style_t *style,
 /*%<
  * Convert all sections of message 'msg' to a cleartext representation
  *
- * Notes on flags:
- *\li	If #DNS_MESSAGETEXTFLAG_NOCOMMENTS is cleared, lines beginning with
- * 	";;" will be emitted indicating section name.
- *\li	If #DNS_MESSAGETEXTFLAG_NOHEADERS is cleared, header lines will be
- * 	emitted.
- *\li   If #DNS_MESSAGETEXTFLAG_ONESOA is set then only print the first
- *	SOA record in the answer section.
- *\li	If *#DNS_MESSAGETEXTFLAG_OMITSOA is set don't print any SOA records
- *	in the answer section.
+ * Notes:
+ * \li     In flags, If #DNS_MESSAGETEXTFLAG_OMITDOT is set, then the
+ *      final '.' in absolute names will not be emitted.  If
+ *      #DNS_MESSAGETEXTFLAG_NOCOMMENTS is cleared, lines beginning
+ *      with ";;" will be emitted indicating section name.  If
+ *      #DNS_MESSAGETEXTFLAG_NOHEADERS is cleared, header lines will
+ *      be emitted.
  *
- * The SOA flags are useful for suppressing the display of the second
- * SOA record in an AXFR by setting #DNS_MESSAGETEXTFLAG_ONESOA on the
- * first message in an AXFR stream and #DNS_MESSAGETEXTFLAG_OMITSOA on
- * subsequent messages.
+ *	If #DNS_MESSAGETEXTFLAG_ONESOA is set then only print the
+ *	first SOA record in the answer section.  If
+ *	#DNS_MESSAGETEXTFLAG_OMITSOA is set don't print any SOA records
+ *	in the answer section.  These are useful for suppressing the
+ *	display of the second SOA record in a AXFR by setting
+ *	#DNS_MESSAGETEXTFLAG_ONESOA on the first message in a AXFR stream
+ *	and #DNS_MESSAGETEXTFLAG_OMITSOA on subsequent messages.
  *
  * Requires:
  *
@@ -600,7 +485,7 @@ dns_message_renderbegin(dns_message_t *msg, dns_compress_t *cctx,
  *
  *\li	'cctx' be valid.
  *
- *\li	'buffer' is a valid buffer with length less than 65536.
+ *\li	'buffer' is a valid buffer.
  *
  * Side Effects:
  *
@@ -805,7 +690,7 @@ dns_message_currentname(dns_message_t *msg, dns_section_t section,
 
 isc_result_t
 dns_message_findname(dns_message_t *msg, dns_section_t section,
-		     const dns_name_t *target, dns_rdatatype_t type,
+		     dns_name_t *target, dns_rdatatype_t type,
 		     dns_rdatatype_t covers, dns_name_t **foundname,
 		     dns_rdataset_t **rdataset);
 /*%<
@@ -816,7 +701,7 @@ dns_message_findname(dns_message_t *msg, dns_section_t section,
  * Requires:
  *\li	'msg' be valid.
  *
- *\li	'section' be a named section.
+ *\li	'section' be a valid section.
  *
  *\li	If a pointer to the name is desired, 'foundname' should be non-NULL.
  *	If it is non-NULL, '*foundname' MUST be NULL.
@@ -840,7 +725,7 @@ dns_message_findname(dns_message_t *msg, dns_section_t section,
  */
 
 isc_result_t
-dns_message_findtype(const dns_name_t *name, dns_rdatatype_t type,
+dns_message_findtype(dns_name_t *name, dns_rdatatype_t type,
 		     dns_rdatatype_t covers, dns_rdataset_t **rdataset);
 /*%<
  * Search the name for the specified type.  If it is found, *rdataset is
@@ -860,7 +745,7 @@ dns_message_findtype(const dns_name_t *name, dns_rdatatype_t type,
  */
 
 isc_result_t
-dns_message_find(const dns_name_t *name, dns_rdataclass_t rdclass,
+dns_message_find(dns_name_t *name, dns_rdataclass_t rdclass,
 		 dns_rdatatype_t type, dns_rdatatype_t covers,
 		 dns_rdataset_t **rdataset);
 /*%<
@@ -880,6 +765,24 @@ dns_message_find(const dns_name_t *name, dns_rdataclass_t rdclass,
  *\li	#ISC_R_NOTFOUND		-- the desired type does not exist.
  */
 
+void
+dns_message_movename(dns_message_t *msg, dns_name_t *name,
+		     dns_section_t fromsection,
+		     dns_section_t tosection);
+/*%<
+ * Move a name from one section to another.
+ *
+ * Requires:
+ *
+ *\li	'msg' be valid.
+ *
+ *\li	'name' must be a name already in 'fromsection'.
+ *
+ *\li	'fromsection' must be a valid section.
+ *
+ *\li	'tosection' must be a valid section.
+ */
+
 void
 dns_message_addname(dns_message_t *msg, dns_name_t *name,
 		    dns_section_t section);
@@ -916,6 +819,7 @@ dns_message_removename(dns_message_t *msg, dns_name_t *name,
  *\li	'section' be a named section.
  */
 
+
 /*
  * LOANOUT FUNCTIONS
  *
@@ -924,7 +828,7 @@ dns_message_removename(dns_message_t *msg, dns_name_t *name,
  * reset, and must NOT be used after these operations.
  */
 
-void
+isc_result_t
 dns_message_gettempname(dns_message_t *msg, dns_name_t **item);
 /*%<
  * Return a name that can be used for any temporary purpose, including
@@ -932,16 +836,36 @@ dns_message_gettempname(dns_message_t *msg, dns_name_t **item);
  * to the message code using dns_message_puttempname() or inserted into
  * one of the message's sections before the message is destroyed.
  *
- * The name will be associated with a dns_fixedname object, and will
- * be initialized.
+ * It is the caller's responsibility to initialize this name.
  *
  * Requires:
  *\li	msg be a valid message
  *
  *\li	item != NULL && *item == NULL
+ *
+ * Returns:
+ *\li	#ISC_R_SUCCESS		-- All is well.
+ *\li	#ISC_R_NOMEMORY		-- No item can be allocated.
  */
 
-void
+isc_result_t
+dns_message_gettempoffsets(dns_message_t *msg, dns_offsets_t **item);
+/*%<
+ * Return an offsets array that can be used for any temporary purpose,
+ * such as attaching to a temporary name.  The offsets will be freed
+ * when the message is destroyed or reset.
+ *
+ * Requires:
+ *\li	msg be a valid message
+ *
+ *\li	item != NULL && *item == NULL
+ *
+ * Returns:
+ *\li	#ISC_R_SUCCESS		-- All is well.
+ *\li	#ISC_R_NOMEMORY		-- No item can be allocated.
+ */
+
+isc_result_t
 dns_message_gettemprdata(dns_message_t *msg, dns_rdata_t **item);
 /*%<
  * Return a rdata that can be used for any temporary purpose, including
@@ -952,9 +876,13 @@ dns_message_gettemprdata(dns_message_t *msg, dns_rdata_t **item);
  *\li	msg be a valid message
  *
  *\li	item != NULL && *item == NULL
+ *
+ * Returns:
+ *\li	#ISC_R_SUCCESS		-- All is well.
+ *\li	#ISC_R_NOMEMORY		-- No item can be allocated.
  */
 
-void
+isc_result_t
 dns_message_gettemprdataset(dns_message_t *msg, dns_rdataset_t **item);
 /*%<
  * Return a rdataset that can be used for any temporary purpose, including
@@ -966,9 +894,13 @@ dns_message_gettemprdataset(dns_message_t *msg, dns_rdataset_t **item);
  *\li	msg be a valid message
  *
  *\li	item != NULL && *item == NULL
+ *
+ * Returns:
+ *\li	#ISC_R_SUCCESS		-- All is well.
+ *\li	#ISC_R_NOMEMORY		-- No item can be allocated.
  */
 
-void
+isc_result_t
 dns_message_gettemprdatalist(dns_message_t *msg, dns_rdatalist_t **item);
 /*%<
  * Return a rdatalist that can be used for any temporary purpose, including
@@ -979,6 +911,10 @@ dns_message_gettemprdatalist(dns_message_t *msg, dns_rdatalist_t **item);
  *\li	msg be a valid message
  *
  *\li	item != NULL && *item == NULL
+ *
+ * Returns:
+ *\li	#ISC_R_SUCCESS		-- All is well.
+ *\li	#ISC_R_NOMEMORY		-- No item can be allocated.
  */
 
 void
@@ -1066,7 +1002,7 @@ dns_message_peekheader(isc_buffer_t *source, dns_messageid_t *idp,
  */
 
 isc_result_t
-dns_message_reply(dns_message_t *msg, bool want_question_section);
+dns_message_reply(dns_message_t *msg, isc_boolean_t want_question_section);
 /*%<
  * Start formatting a reply to the query in 'msg'.
  *
@@ -1090,7 +1026,7 @@ dns_message_reply(dns_message_t *msg, bool want_question_section);
  *\li	#DNS_R_FORMERR		-- the header or question section of the
  *				   message is invalid, replying is impossible.
  *				   If DNS_R_FORMERR is returned when
- *				   want_question_section is false, then
+ *				   want_question_section is ISC_FALSE, then
  *				   it's the header section that's bad;
  *				   otherwise either of the header or question
  *				   sections may be bad.
@@ -1138,7 +1074,7 @@ dns_message_setopt(dns_message_t *msg, dns_rdataset_t *opt);
  */
 
 dns_rdataset_t *
-dns_message_gettsig(dns_message_t *msg, const dns_name_t **owner);
+dns_message_gettsig(dns_message_t *msg, dns_name_t **owner);
 /*%<
  * Get the TSIG record and owner for 'msg'.
  *
@@ -1187,7 +1123,7 @@ dns_message_gettsigkey(dns_message_t *msg);
  *\li	'msg' is a valid message
  */
 
-void
+isc_result_t
 dns_message_setquerytsig(dns_message_t *msg, isc_buffer_t *querytsig);
 /*%<
  * Indicates that 'querytsig' is the TSIG from the signed query for which
@@ -1200,6 +1136,11 @@ dns_message_setquerytsig(dns_message_t *msg, isc_buffer_t *querytsig);
  *	or NULL
  *
  *\li	'msg' is a valid message
+ *
+ * Returns:
+ *
+ *\li	#ISC_R_SUCCESS
+ *\li	#ISC_R_NOMEMORY
  */
 
 isc_result_t
@@ -1227,7 +1168,7 @@ dns_message_getquerytsig(dns_message_t *msg, isc_mem_t *mctx,
  */
 
 dns_rdataset_t *
-dns_message_getsig0(dns_message_t *msg, const dns_name_t **owner);
+dns_message_getsig0(dns_message_t *msg, dns_name_t **owner);
 /*%<
  * Get the SIG(0) record and owner for 'msg'.
  *
@@ -1309,8 +1250,8 @@ dns_message_signer(dns_message_t *msg, dns_name_t *signer);
  *\li	#ISC_R_NOTFOUND		- no TSIG or SIG(0) record is present in the
  *				  message
  *
- *\li	#DNS_R_TSIGVERIFYFAILURE	- the message was signed by a TSIG, but
- * the signature failed to verify
+ *\li	#DNS_R_TSIGVERIFYFAILURE	- the message was signed by a TSIG, but the
+ *				  signature failed to verify
  *
  *\li	#DNS_R_TSIGERRORSET	- the message was signed by a TSIG and
  *				  verified, but the query was rejected by
@@ -1393,22 +1334,17 @@ dns_message_getrawmessage(dns_message_t *msg);
 
 void
 dns_message_setsortorder(dns_message_t *msg, dns_rdatasetorderfunc_t order,
-			 dns_aclenv_t *env, dns_acl_t *acl,
-			 const dns_aclelement_t *element);
+			 const void *order_arg);
 /*%<
  * Define the order in which RR sets get rendered by
  * dns_message_rendersection() to be the ascending order
  * defined by the integer value returned by 'order' when
- * given each RR and a ns_sortlist_arg_t constructed from 'env',
- * 'acl', and 'element' as arguments.
- *
- * If 'order' is NULL, a default order is used.
+ * given each RR and 'arg' as arguments.  If 'order' and
+ * 'order_arg' are NULL, a default order is used.
  *
  * Requires:
  *\li	msg be a valid message.
- *\li	If 'env' is NULL, 'order' must be NULL.
- *\li	If 'env' is not NULL, 'order' must not be NULL and at least one of
- *	'acl' and 'element' must also not be NULL.
+ *\li	order_arg is NULL if and only if order is NULL.
  */
 
 void
@@ -1432,40 +1368,23 @@ dns_message_gettimeadjust(dns_message_t *msg);
 
 void
 dns_message_logpacket(dns_message_t *message, const char *description,
-		      const isc_sockaddr_t *address,
 		      isc_logcategory_t *category, isc_logmodule_t *module,
 		      int level, isc_mem_t *mctx);
-
 void
 dns_message_logfmtpacket(dns_message_t *message, const char *description,
-			 const isc_sockaddr_t *address,
 			 isc_logcategory_t *category, isc_logmodule_t *module,
 			 const dns_master_style_t *style, int level,
 			 isc_mem_t *mctx);
 /*%<
  * Log 'message' at the specified logging parameters.
- *
- * For dns_message_logpacket and dns_message_logfmtpacket expect the
- * 'description' to end in a newline.
- *
- * For dns_message_logpacket2 and dns_message_logfmtpacket2
- * 'description' will be emitted at the start of the message followed
- * by the formatted address and a newline.
- *
- * Requires:
- * \li   message be a valid.
- * \li   description to be non NULL.
- * \li   address to be non NULL.
- * \li   category to be valid.
- * \li   module to be valid.
- * \li   style to be valid.
- * \li   mctx to be a valid.
+ * 'description' will be emitted at the start of the message and will
+ * normally end with a newline.
  */
 
 isc_result_t
 dns_message_buildopt(dns_message_t *msg, dns_rdataset_t **opt,
-		     unsigned int version, uint16_t udpsize, unsigned int flags,
-		     dns_ednsopt_t *ednsopts, size_t count);
+		     unsigned int version, isc_uint16_t udpsize,
+		     unsigned int flags, dns_ednsopt_t *ednsopts, size_t count);
 /*%<
  * Built a opt record.
  *
@@ -1480,63 +1399,6 @@ dns_message_buildopt(dns_message_t *msg, dns_rdataset_t **opt,
  * \li	 other.
  */
 
-void
-dns_message_setclass(dns_message_t *msg, dns_rdataclass_t rdclass);
-/*%<
- * Set the expected class of records in the response.
- *
- * Requires:
- * \li   msg be a valid message with parsing intent.
- */
-
-void
-dns_message_setpadding(dns_message_t *msg, uint16_t padding);
-/*%<
- * Set the padding block size in the response.
- * 0 means no padding (default).
- *
- * Requires:
- * \li	msg be a valid message.
- */
-
-void
-dns_message_clonebuffer(dns_message_t *msg);
-/*%<
- * Clone the query or saved buffers if they where not cloned
- * when parsing.
- *
- * Requires:
- * \li   msg be a valid message.
- */
-
-isc_result_t
-dns_message_minttl(dns_message_t *msg, const dns_section_t sectionid,
-		   dns_ttl_t *pttl);
-/*%<
- * Get the smallest TTL from the 'sectionid' section of a rendered
- * message.
- *
- * Requires:
- * \li   msg be a valid rendered message;
- * \li   'pttl != NULL'.
- */
-
-isc_result_t
-dns_message_response_minttl(dns_message_t *msg, dns_ttl_t *pttl);
-/*%<
- * Get the smalled TTL from the Answer section of 'msg', or if empty, try
- * the MIN(SOA TTL, SOA MINIMUM) value from an SOA record in the Authority
- * section. If neither of these are set, return ISC_R_NOTFOUND.
- *
- * Requires:
- * \li   msg be a valid rendered message;
- * \li   'pttl != NULL'.
- */
-
-void
-dns_message_createpools(isc_mem_t *mctx, isc_mempool_t **namepoolp,
-			isc_mempool_t **rdspoolp);
-void
-dns_message_destroypools(isc_mempool_t **namepoolp, isc_mempool_t **rdspoolp);
-
 ISC_LANG_ENDDECLS
+
+#endif /* DNS_MESSAGE_H */
