@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004-2007, 2009, 2011-2015, 2017  Internet Systems Consortium, Inc. ("ISC")
+ * Copyright (C) 2004-2007, 2009, 2011-2014  Internet Systems Consortium, Inc. ("ISC")
  * Copyright (C) 1999-2001, 2003  Internet Software Consortium.
  *
  * Permission to use, copy, modify, and/or distribute this software for any
@@ -15,6 +15,8 @@
  * PERFORMANCE OF THIS SOFTWARE.
  */
 
+/* $Id: log.c,v 1.49 2011/10/13 22:48:24 tbox Exp $ */
+
 /*! \file */
 
 /* Principal Authors: DCL */
@@ -44,8 +46,6 @@ LIBDNS_EXTERNAL_DATA isc_logcategory_t dns_categories[] = {
 	{ "edns-disabled", 0 },
 	{ "rpz",	0 },
 	{ "rate-limit",	0 },
-	{ "cname",	0 },
-	{ "spill",	0 },
 	{ NULL, 	0 }
 };
 
@@ -83,7 +83,7 @@ LIBDNS_EXTERNAL_DATA isc_logmodule_t dns_modules[] = {
 	{ "dns/dlz",		0 },
 	{ "dns/dnssec",		0 },
 	{ "dns/crypto",		0 },
-	{ "dns/ssu",		0 },
+	{ "dns/packets",	0 },
 	{ NULL, 		0 }
 };
 
