@@ -1,9 +1,18 @@
 /*
- * Copyright (C) 1998-2017  Internet Systems Consortium, Inc. ("ISC")
+ * Copyright (C) 2004-2014  Internet Systems Consortium, Inc. ("ISC")
+ * Copyright (C) 1998-2003  Internet Software Consortium.
  *
- * This Source Code Form is subject to the terms of the Mozilla Public
- * License, v. 2.0. If a copy of the MPL was not distributed with this
- * file, You can obtain one at http://mozilla.org/MPL/2.0/.
+ * Permission to use, copy, modify, and/or distribute this software for any
+ * purpose with or without fee is hereby granted, provided that the above
+ * copyright notice and this permission notice appear in all copies.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS" AND ISC DISCLAIMS ALL WARRANTIES WITH
+ * REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
+ * AND FITNESS.  IN NO EVENT SHALL ISC BE LIABLE FOR ANY SPECIAL, DIRECT,
+ * INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
+ * LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE
+ * OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
+ * PERFORMANCE OF THIS SOFTWARE.
  */
 
 /* $Id$ */
@@ -20,7 +29,6 @@
 #include <isc/mem.h>
 #include <isc/once.h>
 #include <isc/print.h>
-#include <isc/random.h>
 #include <isc/string.h>
 #include <isc/thread.h>
 #include <isc/util.h>
@@ -45,6 +53,7 @@ typedef enum {
 typedef enum {
 	fw_start = 0,
 	fw_ordinary,
+	fw_copy,
 	fw_newcurrent
 } fw_state;
 
@@ -105,17 +114,17 @@ static unsigned char maptolower[] = {
 #define CONVERTTOASCII(c)
 #define CONVERTFROMASCII(c)
 
-#define INIT_OFFSETS(name, var, default_offsets) \
-	if ((name)->offsets != NULL)		 \
-		var = (name)->offsets;		 \
+#define INIT_OFFSETS(name, var, default) \
+	if (name->offsets != NULL) \
+		var = name->offsets; \
 	else \
-		var = (default_offsets);
+		var = default;
 
-#define SETUP_OFFSETS(name, var, default_offsets) \
-	if ((name)->offsets != NULL)		  \
-		var = (name)->offsets;		  \
+#define SETUP_OFFSETS(name, var, default) \
+	if (name->offsets != NULL) \
+		var = name->offsets; \
 	else { \
-		var = (default_offsets);      \
+		var = default; \
 		set_offsets(name, var, NULL); \
 	}
 
@@ -144,17 +153,34 @@ do { \
  * literal, to avoid compiler warnings about discarding
  * the const attribute of a string.
  */
-static unsigned char root_ndata[] = { "" };
+static unsigned char root_ndata[] = { '\0' };
 static unsigned char root_offsets[] = { 0 };
 
-static dns_name_t root = DNS_NAME_INITABSOLUTE(root_ndata, root_offsets);
+static dns_name_t root =
+{
+	DNS_NAME_MAGIC,
+	root_ndata, 1, 1,
+	DNS_NAMEATTR_READONLY | DNS_NAMEATTR_ABSOLUTE,
+	root_offsets, NULL,
+	{(void *)-1, (void *)-1},
+	{NULL, NULL}
+};
+
+/* XXXDCL make const? */
 LIBDNS_EXTERNAL_DATA dns_name_t *dns_rootname = &root;
 
-static unsigned char wild_ndata[] = { "\001*" };
+static unsigned char wild_ndata[] = { '\001', '*' };
 static unsigned char wild_offsets[] = { 0 };
 
 static dns_name_t wild =
-	DNS_NAME_INITNONABSOLUTE(wild_ndata, wild_offsets);
+{
+	DNS_NAME_MAGIC,
+	wild_ndata, 2, 1,
+	DNS_NAMEATTR_READONLY,
+	wild_offsets, NULL,
+	{(void *)-1, (void *)-1},
+	{NULL, NULL}
+};
 
 /* XXXDCL make const? */
 LIBDNS_EXTERNAL_DATA dns_name_t *dns_wildcardname = &wild;
@@ -453,10 +479,42 @@ dns_name_internalwildcard(const dns_name_t *name) {
 	return (ISC_FALSE);
 }
 
+static inline unsigned int
+name_hash(dns_name_t *name, isc_boolean_t case_sensitive) {
+	unsigned int length;
+	const unsigned char *s;
+	unsigned int h = 0;
+	unsigned char c;
+
+	length = name->length;
+	if (length > 16)
+		length = 16;
+
+	/*
+	 * This hash function is similar to the one Ousterhout
+	 * uses in Tcl.
+	 */
+	s = name->ndata;
+	if (case_sensitive) {
+		while (length > 0) {
+			h += ( h << 3 ) + *s;
+			s++;
+			length--;
+		}
+	} else {
+		while (length > 0) {
+			c = maptolower[*s];
+			h += ( h << 3 ) + c;
+			s++;
+			length--;
+		}
+	}
+
+	return (h);
+}
+
 unsigned int
 dns_name_hash(dns_name_t *name, isc_boolean_t case_sensitive) {
-	unsigned int length;
-
 	/*
 	 * Provide a hash value for 'name'.
 	 */
@@ -465,12 +523,7 @@ dns_name_hash(dns_name_t *name, isc_boolean_t case_sensitive) {
 	if (name->labels == 0)
 		return (0);
 
-	length = name->length;
-	if (length > 16)
-		length = 16;
-
-	return (isc_hash_function_reverse(name->ndata, length,
-					  case_sensitive, NULL));
+	return (name_hash(name, case_sensitive));
 }
 
 unsigned int
@@ -483,17 +536,19 @@ dns_name_fullhash(dns_name_t *name, isc_boolean_t case_sensitive) {
 	if (name->labels == 0)
 		return (0);
 
-	return (isc_hash_function_reverse(name->ndata, name->length,
-					  case_sensitive, NULL));
+	return (isc_hash_calc((const unsigned char *)name->ndata,
+			      name->length, case_sensitive));
 }
 
 unsigned int
 dns_fullname_hash(dns_name_t *name, isc_boolean_t case_sensitive) {
 	/*
 	 * This function was deprecated due to the breakage of the name space
-	 * convention.	We only keep this internally to provide binary backward
+	 * convention.  We only keep this internally to provide binary backward
 	 * compatibility.
 	 */
+	REQUIRE(VALID_NAME(name));
+
 	return (dns_name_fullhash(name, case_sensitive));
 }
 
@@ -513,8 +568,7 @@ dns_name_hashbylabel(dns_name_t *name, isc_boolean_t case_sensitive) {
 	if (name->labels == 0)
 		return (0);
 	else if (name->labels == 1)
-		return (isc_hash_function_reverse(name->ndata, name->length,
-						  case_sensitive, NULL));
+		return (name_hash(name, case_sensitive));
 
 	SETUP_OFFSETS(name, offsets, odata);
 	DNS_NAME_INIT(&tname, NULL);
@@ -526,8 +580,7 @@ dns_name_hashbylabel(dns_name_t *name, isc_boolean_t case_sensitive) {
 			tname.length = name->length - offsets[i];
 		else
 			tname.length = offsets[i + 1] - offsets[i];
-		h += isc_hash_function_reverse(tname.ndata, tname.length,
-					       case_sensitive, NULL);
+		h += name_hash(&tname, case_sensitive);
 	}
 
 	return (h);
@@ -565,9 +618,8 @@ dns_name_fullcompare(const dns_name_t *name1, const dns_name_t *name2,
 	REQUIRE((name1->attributes & DNS_NAMEATTR_ABSOLUTE) ==
 		(name2->attributes & DNS_NAMEATTR_ABSOLUTE));
 
-	if (ISC_UNLIKELY(name1 == name2)) {
+	if (name1 == name2) {
 		*orderp = 0;
-		*nlabelsp = name1->labels;
 		return (dns_namereln_equal);
 	}
 
@@ -585,15 +637,12 @@ dns_name_fullcompare(const dns_name_t *name1, const dns_name_t *name2,
 		ldiff = l1 - l2;
 	}
 
-	offsets1 += l1;
-	offsets2 += l2;
-
-	while (ISC_LIKELY(l > 0)) {
+	while (l > 0) {
 		l--;
-		offsets1--;
-		offsets2--;
-		label1 = &name1->ndata[*offsets1];
-		label2 = &name2->ndata[*offsets2];
+		l1--;
+		l2--;
+		label1 = &name1->ndata[offsets1[l1]];
+		label2 = &name2->ndata[offsets2[l2]];
 		count1 = *label1++;
 		count2 = *label2++;
 
@@ -609,43 +658,16 @@ dns_name_fullcompare(const dns_name_t *name1, const dns_name_t *name2,
 		else
 			count = count2;
 
-		/* Loop unrolled for performance */
-		while (ISC_LIKELY(count > 3)) {
-			chdiff = (int)maptolower[label1[0]] -
-				 (int)maptolower[label2[0]];
-			if (chdiff != 0) {
-				*orderp = chdiff;
-				goto done;
-			}
-			chdiff = (int)maptolower[label1[1]] -
-				 (int)maptolower[label2[1]];
-			if (chdiff != 0) {
-				*orderp = chdiff;
-				goto done;
-			}
-			chdiff = (int)maptolower[label1[2]] -
-				 (int)maptolower[label2[2]];
-			if (chdiff != 0) {
-				*orderp = chdiff;
-				goto done;
-			}
-			chdiff = (int)maptolower[label1[3]] -
-				 (int)maptolower[label2[3]];
-			if (chdiff != 0) {
-				*orderp = chdiff;
-				goto done;
-			}
-			count -= 4;
-			label1 += 4;
-			label2 += 4;
-		}
-		while (ISC_LIKELY(count-- > 0)) {
-			chdiff = (int)maptolower[*label1++] -
-				 (int)maptolower[*label2++];
+		while (count > 0) {
+			chdiff = (int)maptolower[*label1] -
+			    (int)maptolower[*label2];
 			if (chdiff != 0) {
 				*orderp = chdiff;
 				goto done;
 			}
+			count--;
+			label1++;
+			label2++;
 		}
 		if (cdiff != 0) {
 			*orderp = cdiff;
@@ -661,12 +683,11 @@ dns_name_fullcompare(const dns_name_t *name1, const dns_name_t *name2,
 		namereln = dns_namereln_subdomain;
 	else
 		namereln = dns_namereln_equal;
-	*nlabelsp = nlabels;
-	return (namereln);
 
  done:
 	*nlabelsp = nlabels;
-	if (nlabels > 0)
+
+	if (nlabels > 0 && namereln == dns_namereln_none)
 		namereln = dns_namereln_commonancestor;
 
 	return (namereln);
@@ -715,7 +736,7 @@ dns_name_equal(const dns_name_t *name1, const dns_name_t *name2) {
 	REQUIRE((name1->attributes & DNS_NAMEATTR_ABSOLUTE) ==
 		(name2->attributes & DNS_NAMEATTR_ABSOLUTE));
 
-	if (ISC_UNLIKELY(name1 == name2))
+	if (name1 == name2)
 		return (ISC_TRUE);
 
 	if (name1->length != name2->length)
@@ -728,32 +749,16 @@ dns_name_equal(const dns_name_t *name1, const dns_name_t *name2) {
 
 	label1 = name1->ndata;
 	label2 = name2->ndata;
-	while (ISC_LIKELY(l-- > 0)) {
+	while (l > 0) {
+		l--;
 		count = *label1++;
 		if (count != *label2++)
 			return (ISC_FALSE);
 
 		INSIST(count <= 63); /* no bitstring support */
 
-		/* Loop unrolled for performance */
-		while (ISC_LIKELY(count > 3)) {
-			c = maptolower[label1[0]];
-			if (c != maptolower[label2[0]])
-				return (ISC_FALSE);
-			c = maptolower[label1[1]];
-			if (c != maptolower[label2[1]])
-				return (ISC_FALSE);
-			c = maptolower[label1[2]];
-			if (c != maptolower[label2[2]])
-				return (ISC_FALSE);
-			c = maptolower[label1[3]];
-			if (c != maptolower[label2[3]])
-				return (ISC_FALSE);
-			count -= 4;
-			label1 += 4;
-			label2 += 4;
-		}
-		while (ISC_LIKELY(count-- > 0)) {
+		while (count > 0) {
+			count--;
 			c = maptolower[*label1++];
 			if (c != maptolower[*label2++])
 				return (ISC_FALSE);
@@ -939,9 +944,9 @@ dns_name_getlabelsequence(const dns_name_t *source,
 			  unsigned int first, unsigned int n,
 			  dns_name_t *target)
 {
-	unsigned char *p, l;
+	unsigned char *offsets;
+	dns_offsets_t odata;
 	unsigned int firstoffset, endoffset;
-	unsigned int i;
 
 	/*
 	 * Make 'target' refer to the 'n' labels including and following
@@ -954,26 +959,17 @@ dns_name_getlabelsequence(const dns_name_t *source,
 	REQUIRE(n <= source->labels - first); /* note first+n could overflow */
 	REQUIRE(BINDABLE(target));
 
-	p = source->ndata;
-	if (ISC_UNLIKELY(first == source->labels)) {
-		firstoffset = source->length;
-	} else {
-		for (i = 0; i < first; i++) {
-			l = *p;
-			p += l + 1;
-		}
-		firstoffset = p - source->ndata;
-	}
+	SETUP_OFFSETS(source, offsets, odata);
 
-	if (ISC_LIKELY(first + n == source->labels))
+	if (first == source->labels)
+		firstoffset = source->length;
+	else
+		firstoffset = offsets[first];
+
+	if (first + n == source->labels)
 		endoffset = source->length;
-	else {
-		for (i = 0; i < n; i++) {
-			l = *p;
-			p += l + 1;
-		}
-		endoffset = p - source->ndata;
-	}
+	else
+		endoffset = offsets[first + n];
 
 	target->ndata = &source->ndata[firstoffset];
 	target->length = endoffset - firstoffset;
@@ -1045,8 +1041,7 @@ dns_name_fromregion(dns_name_t *name, const isc_region_t *r) {
 		len = (r->length < r2.length) ? r->length : r2.length;
 		if (len > DNS_NAME_MAXWIRE)
 			len = DNS_NAME_MAXWIRE;
-		if (len != 0)
-			memmove(r2.base, r->base, len);
+		memmove(r2.base, r->base, len);
 		name->ndata = r2.base;
 		name->length = len;
 	} else {
@@ -1373,7 +1368,7 @@ totext_filter_proc_key_init(void) {
 #endif
 
 isc_result_t
-dns_name_totext(const dns_name_t *name, isc_boolean_t omit_final_dot,
+dns_name_totext(dns_name_t *name, isc_boolean_t omit_final_dot,
 		isc_buffer_t *target)
 {
 	unsigned int options = DNS_NAME_MASTERFILE;
@@ -1384,13 +1379,12 @@ dns_name_totext(const dns_name_t *name, isc_boolean_t omit_final_dot,
 }
 
 isc_result_t
-dns_name_toprincipal(const dns_name_t *name, isc_buffer_t *target) {
+dns_name_toprincipal(dns_name_t *name, isc_buffer_t *target) {
 	return (dns_name_totext2(name, DNS_NAME_OMITFINALDOT, target));
 }
 
 isc_result_t
-dns_name_totext2(const dns_name_t *name, unsigned int options,
-		 isc_buffer_t *target)
+dns_name_totext2(dns_name_t *name, unsigned int options, isc_buffer_t *target)
 {
 	unsigned char *ndata;
 	char *tdata;
@@ -1550,13 +1544,9 @@ dns_name_totext2(const dns_name_t *name, unsigned int options,
 	if (nlen != 0 && trem == 0)
 		return (ISC_R_NOSPACE);
 
-	if (!saw_root || omit_final_dot) {
+	if (!saw_root || omit_final_dot)
 		trem++;
-		tdata--;
-	}
-	if (trem > 0) {
-		*tdata = 0;
-	}
+
 	isc_buffer_add(target, tlen - trem);
 
 #ifdef ISC_PLATFORM_USETHREADS
@@ -1641,9 +1631,9 @@ dns_name_tofilenametext(dns_name_t *name, isc_boolean_t omit_final_dot,
 					trem--;
 					nlen--;
 				} else {
-					if (trem < 4)
+					if (trem < 3)
 						return (ISC_R_NOSPACE);
-					snprintf(tdata, trem, "%%%02X", c);
+					sprintf(tdata, "%%%02X", c);
 					tdata += 3;
 					trem -= 3;
 					ndata++;
@@ -1766,15 +1756,16 @@ set_offsets(const dns_name_t *name, unsigned char *offsets,
 	offset = 0;
 	nlabels = 0;
 	absolute = ISC_FALSE;
-	while (ISC_LIKELY(offset != length)) {
+	while (offset != length) {
 		INSIST(nlabels < 128);
 		offsets[nlabels++] = offset;
-		count = *ndata;
+		count = *ndata++;
+		offset++;
 		INSIST(count <= 63);
-		offset += count + 1;
-		ndata += count + 1;
+		offset += count;
+		ndata += count;
 		INSIST(offset <= length);
-		if (ISC_UNLIKELY(count == 0)) {
+		if (count == 0) {
 			absolute = ISC_TRUE;
 			break;
 		}
@@ -1908,6 +1899,7 @@ dns_name_fromwire(dns_name_t *name, isc_buffer_t *source,
 				    0)
 					return (DNS_R_DISALLOWED);
 				new_current = c & 0x3F;
+				n = 1;
 				state = fw_newcurrent;
 			} else
 				return (DNS_R_BADLABELTYPE);
@@ -1915,6 +1907,8 @@ dns_name_fromwire(dns_name_t *name, isc_buffer_t *source,
 		case fw_ordinary:
 			if (downcase)
 				c = maptolower[c];
+			/* FALLTHROUGH */
+		case fw_copy:
 			*ndata++ = c;
 			n--;
 			if (n == 0)
@@ -1923,6 +1917,9 @@ dns_name_fromwire(dns_name_t *name, isc_buffer_t *source,
 		case fw_newcurrent:
 			new_current *= 256;
 			new_current += c;
+			n--;
+			if (n != 0)
+				break;
 			if (new_current >= biggest_pointer)
 				return (DNS_R_BADPOINTER);
 			biggest_pointer = new_current;
@@ -2028,11 +2025,8 @@ dns_name_towire(const dns_name_t *name, dns_compress_t *cctx,
 	if (gf) {
 		if (target->length - target->used < gp.length)
 			return (ISC_R_NOSPACE);
-		if (gp.length != 0) {
-			unsigned char *base = target->base;
-			(void)memmove(base + target->used, gp.ndata,
-				      (size_t)gp.length);
-		}
+		(void)memmove((unsigned char *)target->base + target->used,
+			      gp.ndata, (size_t)gp.length);
 		isc_buffer_add(target, gp.length);
 		go |= 0xc000;
 		if (target->length - target->used < 2)
@@ -2043,11 +2037,8 @@ dns_name_towire(const dns_name_t *name, dns_compress_t *cctx,
 	} else {
 		if (target->length - target->used < name->length)
 			return (ISC_R_NOSPACE);
-		if (name->length != 0) {
-			unsigned char *base = target->base;
-			(void)memmove(base + target->used, name->ndata,
-				      (size_t)name->length);
-		}
+		(void)memmove((unsigned char *)target->base + target->used,
+			      name->ndata, (size_t)name->length);
 		isc_buffer_add(target, name->length);
 		dns_compress_add(cctx, name, name, offset);
 	}
@@ -2165,13 +2156,15 @@ dns_name_split(dns_name_t *name, unsigned int suffixlabels,
 
 	REQUIRE(VALID_NAME(name));
 	REQUIRE(suffixlabels > 0);
-	REQUIRE(suffixlabels <= name->labels);
+	REQUIRE(suffixlabels < name->labels);
 	REQUIRE(prefix != NULL || suffix != NULL);
 	REQUIRE(prefix == NULL ||
 		(VALID_NAME(prefix) &&
+		 prefix->buffer != NULL &&
 		 BINDABLE(prefix)));
 	REQUIRE(suffix == NULL ||
 		(VALID_NAME(suffix) &&
+		 suffix->buffer != NULL &&
 		 BINDABLE(suffix)));
 
 	splitlabel = name->labels - suffixlabels;
@@ -2373,8 +2366,7 @@ dns_name_settotextfilter(dns_name_totextfilter_t proc) {
 		return (ISC_R_SUCCESS);
 	}
 	if (proc == NULL) {
-		if (mem != NULL)
-			isc_mem_put(thread_key_mctx, mem, sizeof(*mem));
+		isc_mem_put(thread_key_mctx, mem, sizeof(*mem));
 		res = isc_thread_key_setspecific(totext_filter_proc_key, NULL);
 		if (res != 0)
 			result = ISC_R_UNEXPECTED;
@@ -2397,7 +2389,7 @@ dns_name_settotextfilter(dns_name_totextfilter_t proc) {
 }
 
 void
-dns_name_format(const dns_name_t *name, char *cp, unsigned int size) {
+dns_name_format(dns_name_t *name, char *cp, unsigned int size) {
 	isc_result_t result;
 	isc_buffer_t buf;
 
@@ -2441,8 +2433,6 @@ dns_name_tostring(dns_name_t *name, char **target, isc_mem_t *mctx) {
 
 	isc_buffer_usedregion(&buf, &reg);
 	p = isc_mem_allocate(mctx, reg.length + 1);
-	if (p == NULL)
-		return (ISC_R_NOMEMORY);
 	memmove(p, (char *) reg.base, (int) reg.length);
 	p[reg.length] = '\0';
 
@@ -2492,7 +2482,7 @@ dns_name_fromstring2(dns_name_t *target, const char *src,
 }
 
 isc_result_t
-dns_name_copy(const dns_name_t *source, dns_name_t *dest, isc_buffer_t *target) {
+dns_name_copy(dns_name_t *source, dns_name_t *dest, isc_buffer_t *target) {
 	unsigned char *ndata;
 
 	/*
@@ -2519,8 +2509,7 @@ dns_name_copy(const dns_name_t *source, dns_name_t *dest, isc_buffer_t *target)
 	ndata = (unsigned char *)target->base + target->used;
 	dest->ndata = target->base;
 
-	if (source->length != 0)
-		memmove(ndata, source->ndata, source->length);
+	memmove(ndata, source->ndata, source->length);
 
 	dest->ndata = ndata;
 	dest->labels = source->labels;
@@ -2558,175 +2547,3 @@ dns_name_destroy(void) {
 
 #endif
 }
-
-/*
- * Service Discovery Prefixes RFC 6763.
- */
-static unsigned char b_dns_sd_udp_data[]  = "\001b\007_dns-sd\004_udp";
-static unsigned char b_dns_sd_udp_offsets[] = { 0, 2, 10 };
-static unsigned char db_dns_sd_udp_data[]  = "\002db\007_dns-sd\004_udp";
-static unsigned char db_dns_sd_udp_offsets[] = { 0, 3, 11 };
-static unsigned char r_dns_sd_udp_data[]  = "\001r\007_dns-sd\004_udp";
-static unsigned char r_dns_sd_udp_offsets[] = { 0, 2, 10 };
-static unsigned char dr_dns_sd_udp_data[]  = "\002dr\007_dns-sd\004_udp";
-static unsigned char dr_dns_sd_udp_offsets[] = { 0, 3, 11 };
-static unsigned char lb_dns_sd_udp_data[]  = "\002lb\007_dns-sd\004_udp";
-static unsigned char lb_dns_sd_udp_offsets[] = { 0, 3, 11 };
-
-static dns_name_t const dns_sd[] = {
-	DNS_NAME_INITNONABSOLUTE(b_dns_sd_udp_data, b_dns_sd_udp_offsets),
-	DNS_NAME_INITNONABSOLUTE(db_dns_sd_udp_data, db_dns_sd_udp_offsets),
-	DNS_NAME_INITNONABSOLUTE(r_dns_sd_udp_data, r_dns_sd_udp_offsets),
-	DNS_NAME_INITNONABSOLUTE(dr_dns_sd_udp_data, dr_dns_sd_udp_offsets),
-	DNS_NAME_INITNONABSOLUTE(lb_dns_sd_udp_data, lb_dns_sd_udp_offsets)
-};
-
-isc_boolean_t
-dns_name_isdnssd(const dns_name_t *name) {
-	size_t i;
-	dns_name_t prefix;
-
-	if (dns_name_countlabels(name) > 3U) {
-		dns_name_init(&prefix, NULL);
-		dns_name_getlabelsequence(name, 0, 3, &prefix);
-		for (i = 0; i < (sizeof(dns_sd)/sizeof(dns_sd[0])); i++)
-			if (dns_name_equal(&prefix, &dns_sd[i]))
-				return (ISC_TRUE);
-	}
-
-	return (ISC_FALSE);
-}
-
-static unsigned char inaddr10_offsets[] = { 0, 3, 11, 16 };
-static unsigned char inaddr172_offsets[] = { 0, 3, 7, 15, 20 };
-static unsigned char inaddr192_offsets[] = { 0, 4, 8, 16, 21 };
-
-static unsigned char inaddr10[] = "\00210\007IN-ADDR\004ARPA";
-
-static unsigned char inaddr16172[] = "\00216\003172\007IN-ADDR\004ARPA";
-static unsigned char inaddr17172[] = "\00217\003172\007IN-ADDR\004ARPA";
-static unsigned char inaddr18172[] = "\00218\003172\007IN-ADDR\004ARPA";
-static unsigned char inaddr19172[] = "\00219\003172\007IN-ADDR\004ARPA";
-static unsigned char inaddr20172[] = "\00220\003172\007IN-ADDR\004ARPA";
-static unsigned char inaddr21172[] = "\00221\003172\007IN-ADDR\004ARPA";
-static unsigned char inaddr22172[] = "\00222\003172\007IN-ADDR\004ARPA";
-static unsigned char inaddr23172[] = "\00223\003172\007IN-ADDR\004ARPA";
-static unsigned char inaddr24172[] = "\00224\003172\007IN-ADDR\004ARPA";
-static unsigned char inaddr25172[] = "\00225\003172\007IN-ADDR\004ARPA";
-static unsigned char inaddr26172[] = "\00226\003172\007IN-ADDR\004ARPA";
-static unsigned char inaddr27172[] = "\00227\003172\007IN-ADDR\004ARPA";
-static unsigned char inaddr28172[] = "\00228\003172\007IN-ADDR\004ARPA";
-static unsigned char inaddr29172[] = "\00229\003172\007IN-ADDR\004ARPA";
-static unsigned char inaddr30172[] = "\00230\003172\007IN-ADDR\004ARPA";
-static unsigned char inaddr31172[] = "\00231\003172\007IN-ADDR\004ARPA";
-
-static unsigned char inaddr168192[] = "\003168\003192\007IN-ADDR\004ARPA";
-
-static dns_name_t const rfc1918names[] = {
-	DNS_NAME_INITABSOLUTE(inaddr10, inaddr10_offsets),
-	DNS_NAME_INITABSOLUTE(inaddr16172, inaddr172_offsets),
-	DNS_NAME_INITABSOLUTE(inaddr17172, inaddr172_offsets),
-	DNS_NAME_INITABSOLUTE(inaddr18172, inaddr172_offsets),
-	DNS_NAME_INITABSOLUTE(inaddr19172, inaddr172_offsets),
-	DNS_NAME_INITABSOLUTE(inaddr20172, inaddr172_offsets),
-	DNS_NAME_INITABSOLUTE(inaddr21172, inaddr172_offsets),
-	DNS_NAME_INITABSOLUTE(inaddr22172, inaddr172_offsets),
-	DNS_NAME_INITABSOLUTE(inaddr23172, inaddr172_offsets),
-	DNS_NAME_INITABSOLUTE(inaddr24172, inaddr172_offsets),
-	DNS_NAME_INITABSOLUTE(inaddr25172, inaddr172_offsets),
-	DNS_NAME_INITABSOLUTE(inaddr26172, inaddr172_offsets),
-	DNS_NAME_INITABSOLUTE(inaddr27172, inaddr172_offsets),
-	DNS_NAME_INITABSOLUTE(inaddr28172, inaddr172_offsets),
-	DNS_NAME_INITABSOLUTE(inaddr29172, inaddr172_offsets),
-	DNS_NAME_INITABSOLUTE(inaddr30172, inaddr172_offsets),
-	DNS_NAME_INITABSOLUTE(inaddr31172, inaddr172_offsets),
-	DNS_NAME_INITABSOLUTE(inaddr168192, inaddr192_offsets)
-};
-
-isc_boolean_t
-dns_name_isrfc1918(const dns_name_t *name) {
-	size_t i;
-
-	for (i = 0; i < (sizeof(rfc1918names)/sizeof(*rfc1918names)); i++)
-		if (dns_name_issubdomain(name, &rfc1918names[i]))
-			return (ISC_TRUE);
-	return (ISC_FALSE);
-}
-
-static unsigned char ulaoffsets[] = { 0, 2, 4, 8, 13 };
-static unsigned char ip6fc[] = "\001c\001f\003ip6\004ARPA";
-static unsigned char ip6fd[] = "\001d\001f\003ip6\004ARPA";
-
-static dns_name_t const ulanames[] = {
-	DNS_NAME_INITABSOLUTE(ip6fc, ulaoffsets),
-	DNS_NAME_INITABSOLUTE(ip6fd, ulaoffsets)
-};
-
-isc_boolean_t
-dns_name_isula(const dns_name_t *name) {
-	size_t i;
-
-	for (i = 0; i < (sizeof(ulanames)/sizeof(*ulanames)); i++)
-		if (dns_name_issubdomain(name, &ulanames[i]))
-			return (ISC_TRUE);
-	return (ISC_FALSE);
-}
-
-/*
- * Use a simple table as we don't want all the locale stuff
- * associated with ishexdigit().
- */
-const char
-ishex[256] = {
-     0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
-     0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
-     0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
-     1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0,
-     0, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0,
-     0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
-     0, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0,
-     0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
-};
-
-isc_boolean_t
-dns_name_istat(const dns_name_t *name) {
-	unsigned char len;
-	const unsigned char *ndata;
-
-	REQUIRE(VALID_NAME(name));
-
-	if (name->labels < 1)
-		return (ISC_FALSE);
-
-	ndata = name->ndata;
-	len = ndata[0];
-	INSIST(len <= name->length);
-	ndata++;
-
-	/*
-	 * Is there at least one trust anchor reported and is the
-	 * label length consistent with a trust-anchor-telementry label.
-	 */
-	if ((len < 8) || (len - 3) % 5 != 0) {
-		return (ISC_FALSE);
-	}
-
-	if (ndata[0] != '_' ||
-	    maptolower[ndata[1]] != 't' ||
-	    maptolower[ndata[2]] != 'a') {
-		return (ISC_FALSE);
-	}
-	ndata += 3;
-	len -= 3;
-
-	while (len > 0) {
-		INSIST(len >= 5);
-		if (ndata[0] != '-' || !ishex[ndata[1]] || !ishex[ndata[2]] ||
-		    !ishex[ndata[3]] || !ishex[ndata[4]]) {
-			return (ISC_FALSE);
-		}
-		ndata += 5;
-		len -= 5;
-	}
-	return (ISC_TRUE);
-}
