@@ -1,5 +1,5 @@
 /*
- * Portions Copyright (C) 2004-2009, 2011-2017  Internet Systems Consortium, Inc. ("ISC")
+ * Portions Copyright (C) 2004-2009, 2011-2014  Internet Systems Consortium, Inc. ("ISC")
  * Portions Copyright (C) 1999-2002  Internet Software Consortium.
  *
  * Permission to use, copy, modify, and/or distribute this software for any
@@ -41,7 +41,6 @@
 #include <ctype.h>
 
 #include <isc/mem.h>
-#include <isc/safe.h>
 #include <isc/string.h>
 #include <isc/util.h>
 
@@ -72,76 +71,13 @@
 
 static isc_result_t openssldh_todns(const dst_key_t *key, isc_buffer_t *data);
 
-static BIGNUM *bn2, *bn768, *bn1024, *bn1536;
-
-#if OPENSSL_VERSION_NUMBER < 0x10100000L || defined(LIBRESSL_VERSION_NUMBER)
-/*
- * DH_get0_key, DH_set0_key, DH_get0_pqg and DH_set0_pqg
- * are from OpenSSL 1.1.0.
- */
-static void
-DH_get0_key(const DH *dh, const BIGNUM **pub_key, const BIGNUM **priv_key) {
-	if (pub_key != NULL)
-		*pub_key = dh->pub_key;
-	if (priv_key != NULL)
-		*priv_key = dh->priv_key;
-}
-
-static int
-DH_set0_key(DH *dh, BIGNUM *pub_key, BIGNUM *priv_key) {
-	/* Note that it is valid for priv_key to be NULL */
-	if (pub_key == NULL)
-		return 0;
-
-	BN_free(dh->pub_key);
-	BN_free(dh->priv_key);
-	dh->pub_key = pub_key;
-	dh->priv_key = priv_key;
-
-	return 1;
-}
-
-static void
-DH_get0_pqg(const DH *dh,
-	    const BIGNUM **p, const BIGNUM **q, const BIGNUM **g)
-{
-	if (p != NULL)
-		*p = dh->p;
-	if (q != NULL)
-		*q = dh->q;
-	if (g != NULL)
-		*g = dh->g;
-}
-
-static int
-DH_set0_pqg(DH *dh, BIGNUM *p, BIGNUM *q, BIGNUM *g) {
-	/* q is optional */
-	if (p == NULL || g == NULL)
-		return(0);
-	BN_free(dh->p);
-	BN_free(dh->q);
-	BN_free(dh->g);
-	dh->p = p;
-	dh->q = q;
-	dh->g = g;
-
-	if (q != NULL) {
-		dh->length = BN_num_bits(q);
-	}
-
-	return(1);
-}
-
-#define DH_clear_flags(d, f) (d)->flags &= ~(f)
-
-#endif
+static BIGNUM bn2, bn768, bn1024, bn1536;
 
 static isc_result_t
 openssldh_computesecret(const dst_key_t *pub, const dst_key_t *priv,
 			isc_buffer_t *secret)
 {
 	DH *dhpub, *dhpriv;
-	const BIGNUM *pub_key = NULL;
 	int ret;
 	isc_region_t r;
 	unsigned int len;
@@ -156,10 +92,8 @@ openssldh_computesecret(const dst_key_t *pub, const dst_key_t *priv,
 	isc_buffer_availableregion(secret, &r);
 	if (r.length < len)
 		return (ISC_R_NOSPACE);
-
-	DH_get0_key(dhpub, &pub_key, NULL);
-	ret = DH_compute_key(r.base, pub_key, dhpriv);
-	if (ret <= 0)
+	ret = DH_compute_key(r.base, dhpub->pub_key, dhpriv);
+	if (ret == 0)
 		return (dst__openssl_toresult2("DH_compute_key",
 					       DST_R_COMPUTESECRETFAILURE));
 	isc_buffer_add(secret, len);
@@ -168,10 +102,8 @@ openssldh_computesecret(const dst_key_t *pub, const dst_key_t *priv,
 
 static isc_boolean_t
 openssldh_compare(const dst_key_t *key1, const dst_key_t *key2) {
+	int status;
 	DH *dh1, *dh2;
-	const BIGNUM *pub_key1 = NULL, *pub_key2 = NULL;
-	const BIGNUM *priv_key1 = NULL, *priv_key2 = NULL;
-	const BIGNUM *p1 = NULL, *g1 = NULL, *p2 = NULL, *g2 = NULL;
 
 	dh1 = key1->keydata.dh;
 	dh2 = key2->keydata.dh;
@@ -181,19 +113,17 @@ openssldh_compare(const dst_key_t *key1, const dst_key_t *key2) {
 	else if (dh1 == NULL || dh2 == NULL)
 		return (ISC_FALSE);
 
-	DH_get0_key(dh1, &pub_key1, &priv_key1);
-	DH_get0_key(dh2, &pub_key2, &priv_key2);
-	DH_get0_pqg(dh1, &p1, NULL, &g1);
-	DH_get0_pqg(dh2, &p2, NULL, &g2);
+	status = BN_cmp(dh1->p, dh2->p) ||
+		 BN_cmp(dh1->g, dh2->g) ||
+		 BN_cmp(dh1->pub_key, dh2->pub_key);
 
-	if (BN_cmp(p1, p2) != 0 || BN_cmp(g1, g2) != 0 ||
-	    BN_cmp(pub_key1, pub_key2) != 0)
+	if (status != 0)
 		return (ISC_FALSE);
 
-	if (priv_key1 != NULL || priv_key2 != NULL) {
-		if (priv_key1 == NULL || priv_key2 == NULL)
+	if (dh1->priv_key != NULL || dh2->priv_key != NULL) {
+		if (dh1->priv_key == NULL || dh2->priv_key == NULL)
 			return (ISC_FALSE);
-		if (BN_cmp(priv_key1, priv_key2) != 0)
+		if (BN_cmp(dh1->priv_key, dh2->priv_key) != 0)
 			return (ISC_FALSE);
 	}
 	return (ISC_TRUE);
@@ -201,8 +131,8 @@ openssldh_compare(const dst_key_t *key1, const dst_key_t *key2) {
 
 static isc_boolean_t
 openssldh_paramcompare(const dst_key_t *key1, const dst_key_t *key2) {
+	int status;
 	DH *dh1, *dh2;
-	const BIGNUM *p1 = NULL, *g1 = NULL, *p2 = NULL, *g2 = NULL;
 
 	dh1 = key1->keydata.dh;
 	dh2 = key2->keydata.dh;
@@ -212,17 +142,18 @@ openssldh_paramcompare(const dst_key_t *key1, const dst_key_t *key2) {
 	else if (dh1 == NULL || dh2 == NULL)
 		return (ISC_FALSE);
 
-	DH_get0_pqg(dh1, &p1, NULL, &g1);
-	DH_get0_pqg(dh2, &p2, NULL, &g2);
+	status = BN_cmp(dh1->p, dh2->p) ||
+		 BN_cmp(dh1->g, dh2->g);
 
-	if (BN_cmp(p1, p2) != 0 || BN_cmp(g1, g2) != 0)
+	if (status != 0)
 		return (ISC_FALSE);
 	return (ISC_TRUE);
 }
 
 #if OPENSSL_VERSION_NUMBER > 0x00908000L
 static int
-progress_cb(int p, int n, BN_GENCB *cb) {
+progress_cb(int p, int n, BN_GENCB *cb)
+{
 	union {
 		void *dptr;
 		void (*fptr)(int);
@@ -230,7 +161,7 @@ progress_cb(int p, int n, BN_GENCB *cb) {
 
 	UNUSED(n);
 
-	u.dptr = BN_GENCB_get_arg(cb);
+	u.dptr = cb->arg;
 	if (u.fptr != NULL)
 		u.fptr(p);
 	return (1);
@@ -241,10 +172,7 @@ static isc_result_t
 openssldh_generate(dst_key_t *key, int generator, void (*callback)(int)) {
 	DH *dh = NULL;
 #if OPENSSL_VERSION_NUMBER > 0x00908000L
-	BN_GENCB *cb;
-#if OPENSSL_VERSION_NUMBER < 0x10100000L || defined(LIBRESSL_VERSION_NUMBER)
-	BN_GENCB _cb;
-#endif
+	BN_GENCB cb;
 	union {
 		void *dptr;
 		void (*fptr)(int);
@@ -259,25 +187,16 @@ openssldh_generate(dst_key_t *key, int generator, void (*callback)(int)) {
 		    key->key_size == 1024 ||
 		    key->key_size == 1536)
 		{
-			BIGNUM *p, *g;
 			dh = DH_new();
-			if (key->key_size == 768)
-				p = BN_dup(bn768);
-			else if (key->key_size == 1024)
-				p = BN_dup(bn1024);
-			else
-				p = BN_dup(bn1536);
-			g = BN_dup(bn2);
-			if (dh == NULL || p == NULL || g == NULL) {
-				if (dh != NULL)
-					DH_free(dh);
-				if (p != NULL)
-					BN_free(p);
-				if (g != NULL)
-					BN_free(g);
+			if (dh == NULL)
 				return (dst__openssl_toresult(ISC_R_NOMEMORY));
-			}
-			DH_set0_pqg(dh, p, NULL, g);
+			if (key->key_size == 768)
+				dh->p = &bn768;
+			else if (key->key_size == 1024)
+				dh->p = &bn1024;
+			else
+				dh->p = &bn1536;
+			dh->g = &bn2;
 		} else
 			generator = 2;
 	}
@@ -287,45 +206,38 @@ openssldh_generate(dst_key_t *key, int generator, void (*callback)(int)) {
 		dh = DH_new();
 		if (dh == NULL)
 			return (dst__openssl_toresult(ISC_R_NOMEMORY));
-		cb = BN_GENCB_new();
-#if OPENSSL_VERSION_NUMBER >= 0x10100000L && !defined(LIBRESSL_VERSION_NUMBER)
-		if (cb == NULL) {
-			DH_free(dh);
-			return (dst__openssl_toresult(ISC_R_NOMEMORY));
-		}
-#endif
+
 		if (callback == NULL) {
-			BN_GENCB_set_old(cb, NULL, NULL);
+			BN_GENCB_set_old(&cb, NULL, NULL);
 		} else {
 			u.fptr = callback;
-			BN_GENCB_set(cb, &progress_cb, u.dptr);
+			BN_GENCB_set(&cb, &progress_cb, u.dptr);
 		}
 
 		if (!DH_generate_parameters_ex(dh, key->key_size, generator,
-					       cb)) {
+					       &cb)) {
 			DH_free(dh);
-			BN_GENCB_free(cb);
 			return (dst__openssl_toresult2(
 					"DH_generate_parameters_ex",
 					DST_R_OPENSSLFAILURE));
 		}
-		BN_GENCB_free(cb);
 #else
 		dh = DH_generate_parameters(key->key_size, generator,
 					    NULL, NULL);
-		if (dh == NULL)
-			return (dst__openssl_toresult2(
-					"DH_generate_parameters",
-					DST_R_OPENSSLFAILURE));
 #endif
 	}
 
+	if (dh == NULL)
+		return (dst__openssl_toresult2("DH_generate_parameters",
+					       DST_R_OPENSSLFAILURE));
+
 	if (DH_generate_key(dh) == 0) {
 		DH_free(dh);
 		return (dst__openssl_toresult2("DH_generate_key",
 					       DST_R_OPENSSLFAILURE));
 	}
-	DH_clear_flags(dh, DH_FLAG_CACHE_MONT_P);
+	dh->flags &= ~DH_FLAG_CACHE_MONT_P;
+
 	key->keydata.dh = dh;
 
 	return (ISC_R_SUCCESS);
@@ -334,10 +246,7 @@ openssldh_generate(dst_key_t *key, int generator, void (*callback)(int)) {
 static isc_boolean_t
 openssldh_isprivate(const dst_key_t *key) {
 	DH *dh = key->keydata.dh;
-	const BIGNUM *priv_key = NULL;
-
-	DH_get0_key(dh, NULL, &priv_key);
-	return (ISC_TF(dh != NULL && priv_key != NULL));
+	return (ISC_TF(dh != NULL && dh->priv_key != NULL));
 }
 
 static void
@@ -347,16 +256,18 @@ openssldh_destroy(dst_key_t *key) {
 	if (dh == NULL)
 		return;
 
+	if (dh->p == &bn768 || dh->p == &bn1024 || dh->p == &bn1536)
+		dh->p = NULL;
+	if (dh->g == &bn2)
+		dh->g = NULL;
 	DH_free(dh);
 	key->keydata.dh = NULL;
 }
 
 static void
 uint16_toregion(isc_uint16_t val, isc_region_t *region) {
-	*region->base = (val & 0xff00) >> 8;
-	isc_region_consume(region, 1);
-	*region->base = (val & 0x00ff);
-	isc_region_consume(region, 1);
+	*region->base++ = (val & 0xff00) >> 8;
+	*region->base++ = (val & 0x00ff);
 }
 
 static isc_uint16_t
@@ -367,15 +278,13 @@ uint16_fromregion(isc_region_t *region) {
 	val = ((unsigned int)(cp[0])) << 8;
 	val |= ((unsigned int)(cp[1]));
 
-	isc_region_consume(region, 2);
-
+	region->base += 2;
 	return (val);
 }
 
 static isc_result_t
 openssldh_todns(const dst_key_t *key, isc_buffer_t *data) {
 	DH *dh;
-	const BIGNUM *pub_key = NULL, *p = NULL, *g = NULL;
 	isc_region_t r;
 	isc_uint16_t dnslen, plen, glen, publen;
 
@@ -385,44 +294,41 @@ openssldh_todns(const dst_key_t *key, isc_buffer_t *data) {
 
 	isc_buffer_availableregion(data, &r);
 
-	DH_get0_pqg(dh, &p, NULL, &g);
-	if (BN_cmp(g, bn2) == 0 &&
-	    (BN_cmp(p, bn768) == 0 ||
-	     BN_cmp(p, bn1024) == 0 ||
-	     BN_cmp(p, bn1536) == 0)) {
+	if (dh->g == &bn2 &&
+	    (dh->p == &bn768 || dh->p == &bn1024 || dh->p == &bn1536)) {
 		plen = 1;
 		glen = 0;
 	}
 	else {
-		plen = BN_num_bytes(p);
-		glen = BN_num_bytes(g);
+		plen = BN_num_bytes(dh->p);
+		glen = BN_num_bytes(dh->g);
 	}
-	DH_get0_key(dh, &pub_key, NULL);
-	publen = BN_num_bytes(pub_key);
+	publen = BN_num_bytes(dh->pub_key);
 	dnslen = plen + glen + publen + 6;
 	if (r.length < (unsigned int) dnslen)
 		return (ISC_R_NOSPACE);
 
 	uint16_toregion(plen, &r);
 	if (plen == 1) {
-		if (BN_cmp(p, bn768) == 0)
+		if (dh->p == &bn768)
 			*r.base = 1;
-		else if (BN_cmp(p, bn1024) == 0)
+		else if (dh->p == &bn1024)
 			*r.base = 2;
 		else
 			*r.base = 3;
-	} else
-		BN_bn2bin(p, r.base);
-	isc_region_consume(&r, plen);
+	}
+	else
+		BN_bn2bin(dh->p, r.base);
+	r.base += plen;
 
 	uint16_toregion(glen, &r);
 	if (glen > 0)
-		BN_bn2bin(g, r.base);
-	isc_region_consume(&r, glen);
+		BN_bn2bin(dh->g, r.base);
+	r.base += glen;
 
 	uint16_toregion(publen, &r);
-	BN_bn2bin(pub_key, r.base);
-	isc_region_consume(&r, publen);
+	BN_bn2bin(dh->pub_key, r.base);
+	r.base += publen;
 
 	isc_buffer_add(data, dnslen);
 
@@ -432,7 +338,6 @@ openssldh_todns(const dst_key_t *key, isc_buffer_t *data) {
 static isc_result_t
 openssldh_fromdns(dst_key_t *key, isc_buffer_t *data) {
 	DH *dh;
-	BIGNUM *pub_key = NULL, *p = NULL, *g = NULL;
 	isc_region_t r;
 	isc_uint16_t plen, glen, publen;
 	int special = 0;
@@ -444,7 +349,7 @@ openssldh_fromdns(dst_key_t *key, isc_buffer_t *data) {
 	dh = DH_new();
 	if (dh == NULL)
 		return (dst__openssl_toresult(ISC_R_NOMEMORY));
-	DH_clear_flags(dh, DH_FLAG_CACHE_MONT_P);
+	dh->flags &= ~DH_FLAG_CACHE_MONT_P;
 
 	/*
 	 * Read the prime length.  1 & 2 are table entries, > 16 means a
@@ -464,29 +369,28 @@ openssldh_fromdns(dst_key_t *key, isc_buffer_t *data) {
 		return (DST_R_INVALIDPUBLICKEY);
 	}
 	if (plen == 1 || plen == 2) {
-		if (plen == 1) {
-			special = *r.base;
-			isc_region_consume(&r, 1);
-		} else {
+		if (plen == 1)
+			special = *r.base++;
+		else
 			special = uint16_fromregion(&r);
-		}
 		switch (special) {
 			case 1:
-				p = BN_dup(bn768);
+				dh->p = &bn768;
 				break;
 			case 2:
-				p = BN_dup(bn1024);
+				dh->p = &bn1024;
 				break;
 			case 3:
-				p = BN_dup(bn1536);
+				dh->p = &bn1536;
 				break;
 			default:
 				DH_free(dh);
 				return (DST_R_INVALIDPUBLICKEY);
 		}
-	} else {
-		p = BN_bin2bn(r.base, plen, NULL);
-		isc_region_consume(&r, plen);
+	}
+	else {
+		dh->p = BN_bin2bn(r.base, plen, NULL);
+		r.base += plen;
 	}
 
 	/*
@@ -505,33 +409,27 @@ openssldh_fromdns(dst_key_t *key, isc_buffer_t *data) {
 	}
 	if (special != 0) {
 		if (glen == 0)
-			g = BN_dup(bn2);
+			dh->g = &bn2;
 		else {
-			g = BN_bin2bn(r.base, glen, NULL);
-			if (g != NULL && BN_cmp(g, bn2) != 0) {
+			dh->g = BN_bin2bn(r.base, glen, NULL);
+			if (BN_cmp(dh->g, &bn2) == 0) {
+				BN_free(dh->g);
+				dh->g = &bn2;
+			}
+			else {
 				DH_free(dh);
-				BN_free(g);
 				return (DST_R_INVALIDPUBLICKEY);
 			}
 		}
-	} else {
+	}
+	else {
 		if (glen == 0) {
 			DH_free(dh);
 			return (DST_R_INVALIDPUBLICKEY);
 		}
-		g = BN_bin2bn(r.base, glen, NULL);
+		dh->g = BN_bin2bn(r.base, glen, NULL);
 	}
-	isc_region_consume(&r, glen);
-
-	if (p == NULL || g == NULL) {
-		DH_free(dh);
-		if (p != NULL)
-			BN_free(p);
-		if (g != NULL)
-			BN_free(g);
-		return (dst__openssl_toresult(ISC_R_NOMEMORY));
-	}
-	DH_set0_pqg(dh, p, NULL, g);
+	r.base += glen;
 
 	if (r.length < 2) {
 		DH_free(dh);
@@ -542,15 +440,10 @@ openssldh_fromdns(dst_key_t *key, isc_buffer_t *data) {
 		DH_free(dh);
 		return (DST_R_INVALIDPUBLICKEY);
 	}
-	pub_key = BN_bin2bn(r.base, publen, NULL);
-	if (pub_key == NULL) {
-		DH_free(dh);
-		return (dst__openssl_toresult(ISC_R_NOMEMORY));
-	}
-	DH_set0_key(dh, pub_key, NULL);
-	isc_region_consume(&r, publen);
+	dh->pub_key = BN_bin2bn(r.base, publen, NULL);
+	r.base += publen;
 
-	key->key_size = BN_num_bits(p);
+	key->key_size = BN_num_bits(dh->p);
 
 	isc_buffer_forward(data, plen + glen + publen + 6);
 
@@ -563,7 +456,6 @@ static isc_result_t
 openssldh_tofile(const dst_key_t *key, const char *directory) {
 	int i;
 	DH *dh;
-	const BIGNUM *pub_key = NULL, *priv_key = NULL, *p = NULL, *g = NULL;
 	dst_private_t priv;
 	unsigned char *bufs[4];
 	isc_result_t result;
@@ -575,12 +467,10 @@ openssldh_tofile(const dst_key_t *key, const char *directory) {
 		return (DST_R_EXTERNALKEY);
 
 	dh = key->keydata.dh;
-	DH_get0_key(dh, &pub_key, &priv_key);
-	DH_get0_pqg(dh, &p, NULL, &g);
 
 	memset(bufs, 0, sizeof(bufs));
 	for (i = 0; i < 4; i++) {
-		bufs[i] = isc_mem_get(key->mctx, BN_num_bytes(p));
+		bufs[i] = isc_mem_get(key->mctx, BN_num_bytes(dh->p));
 		if (bufs[i] == NULL) {
 			result = ISC_R_NOMEMORY;
 			goto fail;
@@ -590,26 +480,26 @@ openssldh_tofile(const dst_key_t *key, const char *directory) {
 	i = 0;
 
 	priv.elements[i].tag = TAG_DH_PRIME;
-	priv.elements[i].length = BN_num_bytes(p);
-	BN_bn2bin(p, bufs[i]);
+	priv.elements[i].length = BN_num_bytes(dh->p);
+	BN_bn2bin(dh->p, bufs[i]);
 	priv.elements[i].data = bufs[i];
 	i++;
 
 	priv.elements[i].tag = TAG_DH_GENERATOR;
-	priv.elements[i].length = BN_num_bytes(g);
-	BN_bn2bin(g, bufs[i]);
+	priv.elements[i].length = BN_num_bytes(dh->g);
+	BN_bn2bin(dh->g, bufs[i]);
 	priv.elements[i].data = bufs[i];
 	i++;
 
 	priv.elements[i].tag = TAG_DH_PRIVATE;
-	priv.elements[i].length = BN_num_bytes(priv_key);
-	BN_bn2bin(priv_key, bufs[i]);
+	priv.elements[i].length = BN_num_bytes(dh->priv_key);
+	BN_bn2bin(dh->priv_key, bufs[i]);
 	priv.elements[i].data = bufs[i];
 	i++;
 
 	priv.elements[i].tag = TAG_DH_PUBLIC;
-	priv.elements[i].length = BN_num_bytes(pub_key);
-	BN_bn2bin(pub_key, bufs[i]);
+	priv.elements[i].length = BN_num_bytes(dh->pub_key);
+	BN_bn2bin(dh->pub_key, bufs[i]);
 	priv.elements[i].data = bufs[i];
 	i++;
 
@@ -619,7 +509,7 @@ openssldh_tofile(const dst_key_t *key, const char *directory) {
 	for (i = 0; i < 4; i++) {
 		if (bufs[i] == NULL)
 			break;
-		isc_mem_put(key->mctx, bufs[i], BN_num_bytes(p));
+		isc_mem_put(key->mctx, bufs[i], BN_num_bytes(dh->p));
 	}
 	return (result);
 }
@@ -630,7 +520,6 @@ openssldh_parse(dst_key_t *key, isc_lex_t *lexer, dst_key_t *pub) {
 	isc_result_t ret;
 	int i;
 	DH *dh = NULL;
-	BIGNUM *pub_key = NULL, *priv_key = NULL, *p = NULL, *g = NULL;
 	isc_mem_t *mctx;
 #define DST_RET(a) {ret = a; goto err;}
 
@@ -648,7 +537,7 @@ openssldh_parse(dst_key_t *key, isc_lex_t *lexer, dst_key_t *pub) {
 	dh = DH_new();
 	if (dh == NULL)
 		DST_RET(ISC_R_NOMEMORY);
-	DH_clear_flags(dh, DH_FLAG_CACHE_MONT_P);
+	dh->flags &= ~DH_FLAG_CACHE_MONT_P;
 	key->keydata.dh = dh;
 
 	for (i = 0; i < priv.nelements; i++) {
@@ -660,38 +549,54 @@ openssldh_parse(dst_key_t *key, isc_lex_t *lexer, dst_key_t *pub) {
 
 		switch (priv.elements[i].tag) {
 			case TAG_DH_PRIME:
-				p = bn;
+				dh->p = bn;
 				break;
 			case TAG_DH_GENERATOR:
-				g = bn;
+				dh->g = bn;
 				break;
 			case TAG_DH_PRIVATE:
-				priv_key = bn;
+				dh->priv_key = bn;
 				break;
 			case TAG_DH_PUBLIC:
-				pub_key = bn;
+				dh->pub_key = bn;
 				break;
 		}
 	}
 	dst__privstruct_free(&priv, mctx);
-	DH_set0_key(dh, pub_key, priv_key);
-	DH_set0_pqg(dh, p, NULL, g);
 
-	key->key_size = BN_num_bits(p);
+	key->key_size = BN_num_bits(dh->p);
+
+	if ((key->key_size == 768 ||
+	     key->key_size == 1024 ||
+	     key->key_size == 1536) &&
+	    BN_cmp(dh->g, &bn2) == 0)
+	{
+		if (key->key_size == 768 && BN_cmp(dh->p, &bn768) == 0) {
+			BN_free(dh->p);
+			BN_free(dh->g);
+			dh->p = &bn768;
+			dh->g = &bn2;
+		} else if (key->key_size == 1024 &&
+			   BN_cmp(dh->p, &bn1024) == 0) {
+			BN_free(dh->p);
+			BN_free(dh->g);
+			dh->p = &bn1024;
+			dh->g = &bn2;
+		} else if (key->key_size == 1536 &&
+			   BN_cmp(dh->p, &bn1536) == 0) {
+			BN_free(dh->p);
+			BN_free(dh->g);
+			dh->p = &bn1536;
+			dh->g = &bn2;
+		}
+	}
+
 	return (ISC_R_SUCCESS);
 
  err:
-	if (p != NULL)
-		BN_free(p);
-	if (g != NULL)
-		BN_free(g);
-	if (pub_key != NULL)
-		BN_free(pub_key);
-	if (priv_key != NULL)
-		BN_free(priv_key);
 	openssldh_destroy(key);
 	dst__privstruct_free(&priv, mctx);
-	isc_safe_memwipe(&priv, sizeof(priv));
+	memset(&priv, 0, sizeof(priv));
 	return (ret);
 }
 
@@ -704,7 +609,7 @@ BN_fromhex(BIGNUM *b, const char *str) {
 
 	RUNTIME_CHECK(strlen(str) < 1024U && strlen(str) % 2 == 0U);
 	for (i = 0; i < strlen(str); i += 2) {
-		const char *s;
+		char *s;
 		unsigned int high, low;
 
 		s = strchr(hexdigits, tolower((unsigned char)str[i]));
@@ -723,14 +628,15 @@ BN_fromhex(BIGNUM *b, const char *str) {
 
 static void
 openssldh_cleanup(void) {
-	BN_free(bn2);
-	BN_free(bn768);
-	BN_free(bn1024);
-	BN_free(bn1536);
+	BN_free(&bn2);
+	BN_free(&bn768);
+	BN_free(&bn1024);
+	BN_free(&bn1536);
 }
 
 static dst_func_t openssldh_functions = {
 	NULL, /*%< createctx */
+	NULL, /*%< createctx2 */
 	NULL, /*%< destroyctx */
 	NULL, /*%< adddata */
 	NULL, /*%< openssldh_sign */
@@ -756,27 +662,17 @@ isc_result_t
 dst__openssldh_init(dst_func_t **funcp) {
 	REQUIRE(funcp != NULL);
 	if (*funcp == NULL) {
-		bn2 = BN_new();
-		bn768 = BN_new();
-		bn1024 = BN_new();
-		bn1536 = BN_new();
-		if (bn2 == NULL || bn768 == NULL ||
-		    bn1024 == NULL || bn1536 == NULL)
-			goto cleanup;
-		BN_set_word(bn2, 2);
-		BN_fromhex(bn768, PRIME768);
-		BN_fromhex(bn1024, PRIME1024);
-		BN_fromhex(bn1536, PRIME1536);
+		BN_init(&bn2);
+		BN_init(&bn768);
+		BN_init(&bn1024);
+		BN_init(&bn1536);
+		BN_set_word(&bn2, 2);
+		BN_fromhex(&bn768, PRIME768);
+		BN_fromhex(&bn1024, PRIME1024);
+		BN_fromhex(&bn1536, PRIME1536);
 		*funcp = &openssldh_functions;
 	}
 	return (ISC_R_SUCCESS);
-
- cleanup:
-	if (bn2 != NULL) BN_free(bn2);
-	if (bn768 != NULL) BN_free(bn768);
-	if (bn1024 != NULL) BN_free(bn1024);
-	if (bn1536 != NULL) BN_free(bn1536);
-	return (ISC_R_NOMEMORY);
 }
 
 #else /* OPENSSL */
