@@ -1,9 +1,17 @@
 /*
- * Copyright (C) 2008, 2009, 2011, 2012, 2014-2017  Internet Systems Consortium, Inc. ("ISC")
+ * Copyright (C) 2008, 2009, 2011, 2012  Internet Systems Consortium, Inc. ("ISC")
  *
- * This Source Code Form is subject to the terms of the Mozilla Public
- * License, v. 2.0. If a copy of the MPL was not distributed with this
- * file, You can obtain one at http://mozilla.org/MPL/2.0/.
+ * Permission to use, copy, modify, and/or distribute this software for any
+ * purpose with or without fee is hereby granted, provided that the above
+ * copyright notice and this permission notice appear in all copies.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS" AND ISC DISCLAIMS ALL WARRANTIES WITH
+ * REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
+ * AND FITNESS.  IN NO EVENT SHALL ISC BE LIABLE FOR ANY SPECIAL, DIRECT,
+ * INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
+ * LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE
+ * OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
+ * PERFORMANCE OF THIS SOFTWARE.
  */
 
 /* $Id$ */
@@ -37,12 +45,15 @@
 static inline isc_result_t
 fromtext_nsec3(ARGS_FROMTEXT) {
 	isc_token_t token;
+	unsigned char bm[8*1024]; /* 64k bits */
+	dns_rdatatype_t covered;
+	int octet;
+	int window;
 	unsigned int flags;
 	unsigned char hashalg;
 	isc_buffer_t b;
-	unsigned char buf[256];
 
-	REQUIRE(type == dns_rdatatype_nsec3);
+	REQUIRE(type == 50);
 
 	UNUSED(type);
 	UNUSED(rdclass);
@@ -88,26 +99,52 @@ fromtext_nsec3(ARGS_FROMTEXT) {
 	 */
 	RETERR(isc_lex_getmastertoken(lexer, &token, isc_tokentype_string,
 				      ISC_FALSE));
-	isc_buffer_init(&b, buf, sizeof(buf));
-	RETTOK(isc_base32hexnp_decodestring(DNS_AS_STR(token), &b));
+	isc_buffer_init(&b, bm, sizeof(bm));
+	RETTOK(isc_base32hex_decodestring(DNS_AS_STR(token), &b));
 	if (isc_buffer_usedlength(&b) > 0xffU)
 		RETTOK(ISC_R_RANGE);
 	RETERR(uint8_tobuffer(isc_buffer_usedlength(&b), target));
-	RETERR(mem_tobuffer(target, &buf, isc_buffer_usedlength(&b)));
+	RETERR(mem_tobuffer(target, &bm, isc_buffer_usedlength(&b)));
 
-	return (typemap_fromtext(lexer, target, ISC_TRUE));
+	memset(bm, 0, sizeof(bm));
+	do {
+		RETERR(isc_lex_getmastertoken(lexer, &token,
+					      isc_tokentype_string, ISC_TRUE));
+		if (token.type != isc_tokentype_string)
+			break;
+		RETTOK(dns_rdatatype_fromtext(&covered,
+					      &token.value.as_textregion));
+		bm[covered/8] |= (0x80>>(covered%8));
+	} while (1);
+	isc_lex_ungettoken(lexer, &token);
+	for (window = 0; window < 256 ; window++) {
+		/*
+		 * Find if we have a type in this window.
+		 */
+		for (octet = 31; octet >= 0; octet--)
+			if (bm[window * 32 + octet] != 0)
+				break;
+		if (octet < 0)
+			continue;
+		RETERR(uint8_tobuffer(window, target));
+		RETERR(uint8_tobuffer(octet + 1, target));
+		RETERR(mem_tobuffer(target, &bm[window * 32], octet + 1));
+	}
+	return (ISC_R_SUCCESS);
 }
 
 static inline isc_result_t
 totext_nsec3(ARGS_TOTEXT) {
 	isc_region_t sr;
-	unsigned int i, j;
+	unsigned int i, j, k;
+	unsigned int window, len;
 	unsigned char hash;
 	unsigned char flags;
-	char buf[sizeof("TYPE65535")];
+	char buf[sizeof("65535 ")];
 	isc_uint32_t iterations;
+	isc_boolean_t first;
 
-	REQUIRE(rdata->type == dns_rdatatype_nsec3);
+	REQUIRE(rdata->type == 50);
 	REQUIRE(rdata->length != 0);
 
 	dns_rdata_toregion(rdata, &sr);
@@ -115,19 +152,19 @@ totext_nsec3(ARGS_TOTEXT) {
 	/* Hash */
 	hash = uint8_fromregion(&sr);
 	isc_region_consume(&sr, 1);
-	snprintf(buf, sizeof(buf), "%u ", hash);
+	sprintf(buf, "%u ", hash);
 	RETERR(str_totext(buf, target));
 
 	/* Flags */
 	flags = uint8_fromregion(&sr);
 	isc_region_consume(&sr, 1);
-	snprintf(buf, sizeof(buf), "%u ", flags);
+	sprintf(buf, "%u ", flags);
 	RETERR(str_totext(buf, target));
 
 	/* Iterations */
 	iterations = uint16_fromregion(&sr);
 	isc_region_consume(&sr, 2);
-	snprintf(buf, sizeof(buf), "%u ", iterations);
+	sprintf(buf, "%u ", iterations);
 	RETERR(str_totext(buf, target));
 
 	/* Salt */
@@ -154,16 +191,46 @@ totext_nsec3(ARGS_TOTEXT) {
 
 	i = sr.length;
 	sr.length = j;
-	RETERR(isc_base32hexnp_totext(&sr, 1, "", target));
+	RETERR(isc_base32hex_totext(&sr, 1, "", target));
 	sr.length = i - j;
 
-	/*
-	 * Don't leave a trailing space when there's no typemap present.
-	 */
-	if (((tctx->flags & DNS_STYLEFLAG_MULTILINE) == 0) && (sr.length > 0))
+	if ((tctx->flags & DNS_STYLEFLAG_MULTILINE) == 0)
 		RETERR(str_totext(" ", target));
 
-	RETERR(typemap_totext(&sr, tctx, target));
+	/* Types covered */
+	first = ISC_TRUE;
+	for (i = 0; i < sr.length; i += len) {
+		if ((tctx->flags & DNS_STYLEFLAG_MULTILINE) != 0) {
+			RETERR(str_totext(tctx->linebreak, target));
+			first = ISC_TRUE;
+		}
+		INSIST(i + 2 <= sr.length);
+		window = sr.base[i];
+		len = sr.base[i + 1];
+		INSIST(len > 0 && len <= 32);
+		i += 2;
+		INSIST(i + len <= sr.length);
+		for (j = 0; j < len; j++) {
+			dns_rdatatype_t t;
+			if (sr.base[i + j] == 0)
+				continue;
+			for (k = 0; k < 8; k++) {
+				if ((sr.base[i + j] & (0x80 >> k)) == 0)
+					continue;
+				t = window * 256 + j * 8 + k;
+				if (!first)
+					RETERR(str_totext(" ", target));
+				first = ISC_FALSE;
+				if (dns_rdatatype_isknown(t)) {
+					RETERR(dns_rdatatype_totext(t, target));
+				} else {
+					char buf[sizeof("TYPE65535")];
+					sprintf(buf, "TYPE%u", t);
+					RETERR(str_totext(buf, target));
+				}
+			}
+		}
+	}
 
 	if ((tctx->flags & DNS_STYLEFLAG_MULTILINE) != 0)
 		RETERR(str_totext(" )", target));
@@ -174,9 +241,13 @@ totext_nsec3(ARGS_TOTEXT) {
 static inline isc_result_t
 fromwire_nsec3(ARGS_FROMWIRE) {
 	isc_region_t sr, rr;
+	unsigned int window, lastwindow = 0;
+	unsigned int len;
 	unsigned int saltlen, hashlen;
+	isc_boolean_t first = ISC_TRUE;
+	unsigned int i;
 
-	REQUIRE(type == dns_rdatatype_nsec3);
+	REQUIRE(type == 50);
 
 	UNUSED(type);
 	UNUSED(rdclass);
@@ -205,8 +276,40 @@ fromwire_nsec3(ARGS_FROMWIRE) {
 		RETERR(DNS_R_FORMERR);
 	isc_region_consume(&sr, hashlen);
 
-	RETERR(typemap_test(&sr, ISC_TRUE));
-
+	for (i = 0; i < sr.length; i += len) {
+		/*
+		 * Check for overflow.
+		 */
+		if (i + 2 > sr.length)
+			RETERR(DNS_R_FORMERR);
+		window = sr.base[i];
+		len = sr.base[i + 1];
+		i += 2;
+		/*
+		 * Check that bitmap windows are in the correct order.
+		 */
+		if (!first && window <= lastwindow)
+			RETERR(DNS_R_FORMERR);
+		/*
+		 * Check for legal lengths.
+		 */
+		if (len < 1 || len > 32)
+			RETERR(DNS_R_FORMERR);
+		/*
+		 * Check for overflow.
+		 */
+		if (i + len > sr.length)
+			RETERR(DNS_R_FORMERR);
+		/*
+		 * The last octet of the bitmap must be non zero.
+		 */
+		if (sr.base[i + len - 1] == 0)
+			RETERR(DNS_R_FORMERR);
+		lastwindow = window;
+		first = ISC_FALSE;
+	}
+	if (i != sr.length)
+		return (DNS_R_EXTRADATA);
 	RETERR(mem_tobuffer(target, rr.base, rr.length));
 	isc_buffer_forward(source, rr.length);
 	return (ISC_R_SUCCESS);
@@ -216,7 +319,7 @@ static inline isc_result_t
 towire_nsec3(ARGS_TOWIRE) {
 	isc_region_t sr;
 
-	REQUIRE(rdata->type == dns_rdatatype_nsec3);
+	REQUIRE(rdata->type == 50);
 	REQUIRE(rdata->length != 0);
 
 	UNUSED(cctx);
@@ -232,7 +335,7 @@ compare_nsec3(ARGS_COMPARE) {
 
 	REQUIRE(rdata1->type == rdata2->type);
 	REQUIRE(rdata1->rdclass == rdata2->rdclass);
-	REQUIRE(rdata1->type == dns_rdatatype_nsec3);
+	REQUIRE(rdata1->type == 50);
 	REQUIRE(rdata1->length != 0);
 	REQUIRE(rdata2->length != 0);
 
@@ -244,9 +347,10 @@ compare_nsec3(ARGS_COMPARE) {
 static inline isc_result_t
 fromstruct_nsec3(ARGS_FROMSTRUCT) {
 	dns_rdata_nsec3_t *nsec3 = source;
-	isc_region_t region;
+	unsigned int i, len, window, lastwindow = 0;
+	isc_boolean_t first = ISC_TRUE;
 
-	REQUIRE(type == dns_rdatatype_nsec3);
+	REQUIRE(type == 50);
 	REQUIRE(source != NULL);
 	REQUIRE(nsec3->common.rdtype == type);
 	REQUIRE(nsec3->common.rdclass == rdclass);
@@ -264,9 +368,21 @@ fromstruct_nsec3(ARGS_FROMSTRUCT) {
 	RETERR(uint8_tobuffer(nsec3->next_length, target));
 	RETERR(mem_tobuffer(target, nsec3->next, nsec3->next_length));
 
-	region.base = nsec3->typebits;
-	region.length = nsec3->len;
-	RETERR(typemap_test(&region, ISC_TRUE));
+	/*
+	 * Perform sanity check.
+	 */
+	for (i = 0; i < nsec3->len ; i += len) {
+		INSIST(i + 2 <= nsec3->len);
+		window = nsec3->typebits[i];
+		len = nsec3->typebits[i+1];
+		i += 2;
+		INSIST(first || window > lastwindow);
+		INSIST(len > 0 && len <= 32);
+		INSIST(i + len <= nsec3->len);
+		INSIST(nsec3->typebits[i + len - 1] != 0);
+		lastwindow = window;
+		first = ISC_FALSE;
+	}
 	return (mem_tobuffer(target, nsec3->typebits, nsec3->len));
 }
 
@@ -275,7 +391,7 @@ tostruct_nsec3(ARGS_TOSTRUCT) {
 	isc_region_t region;
 	dns_rdata_nsec3_t *nsec3 = target;
 
-	REQUIRE(rdata->type == dns_rdatatype_nsec3);
+	REQUIRE(rdata->type == 50);
 	REQUIRE(target != NULL);
 	REQUIRE(rdata->length != 0);
 
@@ -321,7 +437,7 @@ freestruct_nsec3(ARGS_FREESTRUCT) {
 	dns_rdata_nsec3_t *nsec3 = source;
 
 	REQUIRE(source != NULL);
-	REQUIRE(nsec3->common.rdtype == dns_rdatatype_nsec3);
+	REQUIRE(nsec3->common.rdtype == 50);
 
 	if (nsec3->mctx == NULL)
 		return;
@@ -337,7 +453,7 @@ freestruct_nsec3(ARGS_FREESTRUCT) {
 
 static inline isc_result_t
 additionaldata_nsec3(ARGS_ADDLDATA) {
-	REQUIRE(rdata->type == dns_rdatatype_nsec3);
+	REQUIRE(rdata->type == 50);
 
 	UNUSED(rdata);
 	UNUSED(add);
@@ -350,7 +466,7 @@ static inline isc_result_t
 digest_nsec3(ARGS_DIGEST) {
 	isc_region_t r;
 
-	REQUIRE(rdata->type == dns_rdatatype_nsec3);
+	REQUIRE(rdata->type == 50);
 
 	dns_rdata_toregion(rdata, &r);
 	return ((digest)(arg, &r));
@@ -358,32 +474,21 @@ digest_nsec3(ARGS_DIGEST) {
 
 static inline isc_boolean_t
 checkowner_nsec3(ARGS_CHECKOWNER) {
-	unsigned char owner[NSEC3_MAX_HASH_LENGTH];
-	isc_buffer_t buffer;
-	dns_label_t label;
 
-	REQUIRE(type == dns_rdatatype_nsec3);
+       REQUIRE(type == 50);
 
-	UNUSED(type);
-	UNUSED(rdclass);
-	UNUSED(wildcard);
+       UNUSED(name);
+       UNUSED(type);
+       UNUSED(rdclass);
+       UNUSED(wildcard);
 
-	/*
-	 * First label is a base32hex string without padding.
-	 */
-	dns_name_getlabel(name, 0, &label);
-	isc_region_consume(&label, 1);
-	isc_buffer_init(&buffer, owner, sizeof(owner));
-	if (isc_base32hexnp_decoderegion(&label, &buffer) == ISC_R_SUCCESS)
-		return (ISC_TRUE);
-
-	return (ISC_FALSE);
+       return (ISC_TRUE);
 }
 
 static inline isc_boolean_t
 checknames_nsec3(ARGS_CHECKNAMES) {
 
-	REQUIRE(rdata->type == dns_rdatatype_nsec3);
+	REQUIRE(rdata->type == 50);
 
 	UNUSED(rdata);
 	UNUSED(owner);
