@@ -1,9 +1,17 @@
 /*
- * Copyright (C) 2012-2017  Internet Systems Consortium, Inc. ("ISC")
+ * Copyright (C) 2012-2014  Internet Systems Consortium, Inc. ("ISC")
  *
- * This Source Code Form is subject to the terms of the Mozilla Public
- * License, v. 2.0. If a copy of the MPL was not distributed with this
- * file, You can obtain one at http://mozilla.org/MPL/2.0/.
+ * Permission to use, copy, modify, and/or distribute this software for any
+ * purpose with or without fee is hereby granted, provided that the above
+ * copyright notice and this permission notice appear in all copies.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS" AND ISC DISCLAIMS ALL WARRANTIES WITH
+ * REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
+ * AND FITNESS.  IN NO EVENT SHALL ISC BE LIABLE FOR ANY SPECIAL, DIRECT,
+ * INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
+ * LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE
+ * OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
+ * PERFORMANCE OF THIS SOFTWARE.
  */
 
 /*! \file */
@@ -19,7 +27,6 @@
 #include <isc/net.h>
 #include <isc/netaddr.h>
 #include <isc/print.h>
-#include <isc/util.h>
 
 #include <dns/result.h>
 #include <dns/rcode.h>
@@ -189,18 +196,18 @@ set_age(dns_rrl_t *rrl, dns_rrl_entry_t *e, isc_stdtime_t now) {
 }
 
 static isc_result_t
-expand_entries(dns_rrl_t *rrl, int newsize) {
+expand_entries(dns_rrl_t *rrl, int new) {
 	unsigned int bsize;
 	dns_rrl_block_t *b;
 	dns_rrl_entry_t *e;
 	double rate;
 	int i;
 
-	if (rrl->num_entries + newsize >= rrl->max_entries &&
+	if (rrl->num_entries + new >= rrl->max_entries &&
 	    rrl->max_entries != 0)
 	{
-		newsize = rrl->max_entries - rrl->num_entries;
-		if (newsize <= 0)
+		new = rrl->max_entries - rrl->num_entries;
+		if (new <= 0)
 			return (ISC_R_SUCCESS);
 	}
 
@@ -217,11 +224,11 @@ expand_entries(dns_rrl_t *rrl, int newsize) {
 			      DNS_LOGMODULE_REQUEST, DNS_RRL_LOG_DROP,
 			      "increase from %d to %d RRL entries with"
 			      " %d bins; average search length %.1f",
-			      rrl->num_entries, rrl->num_entries+newsize,
+			      rrl->num_entries, rrl->num_entries+new,
 			      rrl->hash->length, rate);
 	}
 
-	bsize = sizeof(dns_rrl_block_t) + (newsize-1)*sizeof(dns_rrl_entry_t);
+	bsize = sizeof(dns_rrl_block_t) + (new-1)*sizeof(dns_rrl_entry_t);
 	b = isc_mem_get(rrl->mctx, bsize);
 	if (b == NULL) {
 		isc_log_write(dns_lctx, DNS_LOGCATEGORY_RRL,
@@ -234,11 +241,11 @@ expand_entries(dns_rrl_t *rrl, int newsize) {
 	b->size = bsize;
 
 	e = b->entries;
-	for (i = 0; i < newsize; ++i, ++e) {
+	for (i = 0; i < new; ++i, ++e) {
 		ISC_LINK_INIT(e, hlink);
 		ISC_LIST_INITANDAPPEND(rrl->lru, e, lru);
 	}
-	rrl->num_entries += newsize;
+	rrl->num_entries += new;
 	ISC_LIST_INITANDAPPEND(rrl->blocks, b, link);
 
 	return (ISC_R_SUCCESS);
@@ -246,7 +253,6 @@ expand_entries(dns_rrl_t *rrl, int newsize) {
 
 static inline dns_rrl_bin_t *
 get_bin(dns_rrl_hash_t *hash, unsigned int hval) {
-	INSIST(hash != NULL);
 	return (&hash->bins[hval % hash->length]);
 }
 
@@ -369,7 +375,7 @@ hash_key(const dns_rrl_key_t *key) {
 	int i;
 
 	hval = key->w[0];
-	for (i = sizeof(key->w) / sizeof(key->w[0]) - 1; i >= 0; --i) {
+	for (i = sizeof(*key) / sizeof(key->w[0]) - 1; i >= 0; --i) {
 		hval = key->w[i] + (hval<<1);
 	}
 	return (hval);
@@ -418,11 +424,11 @@ make_key(const dns_rrl_t *rrl, dns_rrl_key_t *key,
 		{
 			dns_name_init(&base, base_offsets);
 			dns_name_getlabelsequence(qname, 1, labels-1, &base);
-			key->s.qname_hash =
-				dns_name_fullhash(&base, ISC_FALSE);
+			key->s.qname_hash = dns_name_hashbylabel(&base,
+							ISC_FALSE);
 		} else {
-			key->s.qname_hash =
-				dns_name_fullhash(qname, ISC_FALSE);
+			key->s.qname_hash = dns_name_hashbylabel(qname,
+							ISC_FALSE);
 		}
 	}
 
@@ -1154,17 +1160,22 @@ dns_rrl(dns_view_t *view,
 						 client_addr, now,
 						 log_buf, log_buf_len);
 		if (rrl_all_result != DNS_RRL_RESULT_OK) {
+			int level;
+
 			e = e_all;
 			rrl_result = rrl_all_result;
-			if (isc_log_wouldlog(dns_lctx, DNS_RRL_LOG_DEBUG1)) {
+			if (rrl_result == DNS_RRL_RESULT_OK)
+				level = DNS_RRL_LOG_DEBUG2;
+			else
+				level = DNS_RRL_LOG_DEBUG1;
+			if (isc_log_wouldlog(dns_lctx, level)) {
 				make_log_buf(rrl, e,
 					     "prefer all-per-second limiting ",
 					     NULL, ISC_TRUE, qname, ISC_FALSE,
 					     DNS_RRL_RESULT_OK, resp_result,
 					     log_buf, log_buf_len);
 				isc_log_write(dns_lctx, DNS_LOGCATEGORY_RRL,
-					      DNS_LOGMODULE_REQUEST,
-					      DNS_RRL_LOG_DEBUG1,
+					      DNS_LOGMODULE_REQUEST, level,
 					      "%s", log_buf);
 			}
 		}
