@@ -1,9 +1,18 @@
 /*
- * Copyright (C) 2000, 2001, 2004-2007, 2009, 2013-2018  Internet Systems Consortium, Inc. ("ISC")
+ * Copyright (C) 2004-2007, 2009, 2013, 2014  Internet Systems Consortium, Inc. ("ISC")
+ * Copyright (C) 2000, 2001  Internet Software Consortium.
  *
- * This Source Code Form is subject to the terms of the Mozilla Public
- * License, v. 2.0. If a copy of the MPL was not distributed with this
- * file, You can obtain one at http://mozilla.org/MPL/2.0/.
+ * Permission to use, copy, modify, and/or distribute this software for any
+ * purpose with or without fee is hereby granted, provided that the above
+ * copyright notice and this permission notice appear in all copies.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS" AND ISC DISCLAIMS ALL WARRANTIES WITH
+ * REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
+ * AND FITNESS.  IN NO EVENT SHALL ISC BE LIABLE FOR ANY SPECIAL, DIRECT,
+ * INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
+ * LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE
+ * OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
+ * PERFORMANCE OF THIS SOFTWARE.
  */
 
 /* $Id: hmacmd5.c,v 1.16 2009/02/06 23:47:42 tbox Exp $ */
@@ -15,10 +24,6 @@
 
 #include "config.h"
 
-#include <pk11/site.h>
-
-#ifndef PK11_MD5_DISABLE
-
 #include <isc/assertions.h>
 #include <isc/hmacmd5.h>
 #include <isc/md5.h>
@@ -28,52 +33,39 @@
 #include <isc/types.h>
 #include <isc/util.h>
 
-#if PKCS11CRYPTO
+#if PKCS11CRYPTO || PKCS11CRYPTOWITHHMAC
 #include <pk11/internal.h>
 #include <pk11/pk11.h>
 #endif
 
 #ifdef ISC_PLATFORM_OPENSSLHASH
-#if OPENSSL_VERSION_NUMBER < 0x10100000L || defined(LIBRESSL_VERSION_NUMBER)
-#define HMAC_CTX_new() &(ctx->_ctx), HMAC_CTX_init(&(ctx->_ctx))
-#define HMAC_CTX_free(ptr) HMAC_CTX_cleanup(ptr)
-#endif
 
 void
 isc_hmacmd5_init(isc_hmacmd5_t *ctx, const unsigned char *key,
 		 unsigned int len)
 {
-	ctx->ctx = HMAC_CTX_new();
-	RUNTIME_CHECK(ctx->ctx != NULL);
-	RUNTIME_CHECK(HMAC_Init_ex(ctx->ctx, (const void *) key,
-				   (int) len, EVP_md5(), NULL) == 1);
+	HMAC_Init(ctx, (const void *) key, (int) len, EVP_md5());
 }
 
 void
 isc_hmacmd5_invalidate(isc_hmacmd5_t *ctx) {
-	if (ctx->ctx == NULL)
-		return;
-	HMAC_CTX_free(ctx->ctx);
-	ctx->ctx = NULL;
+	HMAC_CTX_cleanup(ctx);
 }
 
 void
 isc_hmacmd5_update(isc_hmacmd5_t *ctx, const unsigned char *buf,
 		   unsigned int len)
 {
-	RUNTIME_CHECK(HMAC_Update(ctx->ctx, buf, (int) len) == 1);
+	HMAC_Update(ctx, buf, (int) len);
 }
 
 void
 isc_hmacmd5_sign(isc_hmacmd5_t *ctx, unsigned char *digest) {
-	RUNTIME_CHECK(HMAC_Final(ctx->ctx, digest, NULL) == 1);
-	HMAC_CTX_free(ctx->ctx);
-	ctx->ctx = NULL;
+	HMAC_Final(ctx, digest, NULL);
+	HMAC_CTX_cleanup(ctx);
 }
 
-#elif PKCS11CRYPTO
-
-#ifndef PK11_MD5_HMAC_REPLACE
+#elif PKCS11CRYPTOWITHHMAC
 
 static CK_BBOOL truevalue = TRUE;
 static CK_BBOOL falsevalue = FALSE;
@@ -95,19 +87,8 @@ isc_hmacmd5_init(isc_hmacmd5_t *ctx, const unsigned char *key,
 		{ CKA_SIGN, &truevalue, (CK_ULONG) sizeof(truevalue) },
 		{ CKA_VALUE, NULL, (CK_ULONG) len }
 	};
-#ifdef PK11_PAD_HMAC_KEYS
-	CK_BYTE keypad[ISC_MD5_DIGESTLENGTH];
 
-	if (len < ISC_MD5_DIGESTLENGTH) {
-		memset(keypad, 0, ISC_MD5_DIGESTLENGTH);
-		memmove(keypad, key, len);
-		keyTemplate[5].pValue = keypad;
-		keyTemplate[5].ulValueLen = ISC_MD5_DIGESTLENGTH;
-	} else
-		DE_CONST(key, keyTemplate[5].pValue);
-#else
 	DE_CONST(key, keyTemplate[5].pValue);
-#endif
 	RUNTIME_CHECK(pk11_get_session(ctx, OP_DIGEST, ISC_TRUE, ISC_FALSE,
 				       ISC_FALSE, NULL, 0) == ISC_R_SUCCESS);
 	ctx->object = CK_INVALID_HANDLE;
@@ -126,7 +107,7 @@ isc_hmacmd5_invalidate(isc_hmacmd5_t *ctx) {
 	if (ctx->handle == NULL)
 		return;
 	(void) pkcs_C_SignFinal(ctx->session, garbage, &len);
-	isc_safe_memwipe(garbage, sizeof(garbage));
+	memset(garbage, 0, sizeof(garbage));
 	if (ctx->object != CK_INVALID_HANDLE)
 		(void) pkcs_C_DestroyObject(ctx->session, ctx->object);
 	ctx->object = CK_INVALID_HANDLE;
@@ -157,8 +138,8 @@ isc_hmacmd5_sign(isc_hmacmd5_t *ctx, unsigned char *digest) {
 	ctx->object = CK_INVALID_HANDLE;
 	pk11_return_session(ctx);
 }
-#else
-/* Replace missing CKM_MD5_HMAC PKCS#11 mechanism */
+
+#elif PKCS11CRYPTO
 
 #define PADLEN 64
 #define IPAD 0x36
@@ -244,7 +225,6 @@ isc_hmacmd5_sign(isc_hmacmd5_t *ctx, unsigned char *digest) {
 			 (CK_ULONG_PTR) &len));
 	pk11_return_session(ctx);
 }
-#endif
 
 #else
 
@@ -281,7 +261,7 @@ isc_hmacmd5_init(isc_hmacmd5_t *ctx, const unsigned char *key,
 void
 isc_hmacmd5_invalidate(isc_hmacmd5_t *ctx) {
 	isc_md5_invalidate(&ctx->md5ctx);
-	isc_safe_memwipe(ctx->key, sizeof(ctx->key));
+	memset(ctx->key, 0, sizeof(ctx->key));
 }
 
 /*!
@@ -333,86 +313,5 @@ isc_hmacmd5_verify2(isc_hmacmd5_t *ctx, unsigned char *digest, size_t len) {
 
 	REQUIRE(len <= ISC_MD5_DIGESTLENGTH);
 	isc_hmacmd5_sign(ctx, newdigest);
-	return (isc_safe_memequal(digest, newdigest, len));
+	return (isc_safe_memcmp(digest, newdigest, len));
 }
-
-/*
- * Check for MD5 support; if it does not work, raise a fatal error.
- *
- * Use the first test vector from RFC 2104, with a second round using
- * a too-short key.
- *
- * Standard use is testing 0 and expecting result true.
- * Testing use is testing 1..4 and expecting result false.
- */
-isc_boolean_t
-isc_hmacmd5_check(int testing) {
-	isc_hmacmd5_t ctx;
-	unsigned char key[] = { /* 0x0b0b0b0b0b0b0b0b0b0b0b0b0b0b0b0b */
-		0x0b, 0x0b, 0x0b, 0x0b, 0x0b, 0x0b, 0x0b, 0x0b,
-		0x0b, 0x0b, 0x0b, 0x0b, 0x0b, 0x0b, 0x0b, 0x0b
-	};
-	unsigned char input[] = { /* "Hi There" */
-		0x48, 0x69, 0x20, 0x54, 0x68, 0x65, 0x72, 0x65
-	};
-	unsigned char expected[] = {
-		0x92, 0x94, 0x72, 0x7a, 0x36, 0x38, 0xbb, 0x1c,
-		0x13, 0xf4, 0x8e, 0xf8, 0x15, 0x8b, 0xfc, 0x9d
-	};
-	unsigned char expected2[] = {
-		0xad, 0xb8, 0x48, 0x05, 0xb8, 0x8d, 0x03, 0xe5,
-		0x90, 0x1e, 0x4b, 0x05, 0x69, 0xce, 0x35, 0xea
-	};
-	isc_boolean_t result;
-
-	/*
-	 * Introduce a fault for testing.
-	 */
-	switch (testing) {
-	case 0:
-	default:
-		break;
-	case 1:
-		key[0] ^= 0x01;
-		break;
-	case 2:
-		input[0] ^= 0x01;
-		break;
-	case 3:
-		expected[0] ^= 0x01;
-		break;
-	case 4:
-		expected2[0] ^= 0x01;
-		break;
-	}
-
-	/*
-	 * These functions do not return anything; any failure will be fatal.
-	 */
-	isc_hmacmd5_init(&ctx, key, 16U);
-	isc_hmacmd5_update(&ctx, input, 8U);
-	result = isc_hmacmd5_verify2(&ctx, expected, sizeof(expected));
-	if (!result) {
-		return (result);
-	}
-
-	/* Second round using a byte key */
-	isc_hmacmd5_init(&ctx, key, 1U);
-	isc_hmacmd5_update(&ctx, input, 8U);
-	return (isc_hmacmd5_verify2(&ctx, expected2, sizeof(expected2)));
-}
-
-#else /* !PK11_MD5_DISABLE */
-#ifdef WIN32
-/* Make the Visual Studio linker happy */
-#include <isc/util.h>
-
-void isc_hmacmd5_init() { INSIST(0); }
-void isc_hmacmd5_invalidate() { INSIST(0); }
-void isc_hmacmd5_sign() { INSIST(0); }
-void isc_hmacmd5_update() { INSIST(0); }
-void isc_hmacmd5_verify() { INSIST(0); }
-void isc_hmacmd5_verify2() { INSIST(0); }
-void isc_hmacmd5_check() { INSIST(0); }
-#endif
-#endif /* PK11_MD5_DISABLE */
